<?php

// Pfad zu include Dateien bestimmen
file_exists ( "config/" ) ? $include_prefix = "" : $include_prefix = "../";

if (! class_exists ( "Liste" )) {
	class Liste {
		public $sListeName;
		protected $aSpalten = array ();
		protected $aZeilen = array ();
		protected $bEigenesDrucken = false;
		protected $bAusrichtungBestimmt = false;
		
		/**
		 * Konstruktor
		 *
		 * @param string $sListeName        	
		 * @param bool $bEigenesDrucken
		 *        	- Wenn true wird das Target drucken.php verwendet
		 */
		public function __construct($sListeName, $bEigenesDrucken = false) {
			global $listenname;
			
			$listenname = $sListeName;
			$this->sListeName = $sListeName;
			$this->bEigenesDrucken = $bEigenesDrucken;
			
			if ($this->bEigenesDrucken) {
				if (! check_link_permission ( $GLOBALS ['target'], "drucken", $GLOBALS ['benutzerrechte'], 0 )) {
					$this->bEigenesDrucken = false;
				}
			}
		}
		
		/**
		 * Fgt eine Listenspalte hinzu
		 *
		 * @param Listenspalte $oListenspalte        	
		 */
		public function addSpalte($oListenspalte) {
			$this->aSpalten [] = $oListenspalte;
		}
		
		/**
		 * Fgt ein fertiges Listenarray hinzu
		 *
		 * @param array $aOutput        	
		 */
		public function addZeilen($aOutput) {
			if (! is_array ( $aOutput )) {
				return false;
			}
			
			$this->aZeilen = array_merge ( $this->aZeilen, $aOutput );
		}
		public function getAnzahlZeilen() {
			return count ( $this->aZeilen );
		}
		public function getAnzahlZeilenAusgabe() {
			return count ( $this->aZeilen ) - $this->iAnzahlZeilenBlind;
		}
		public function setAnzahlZeilenBlind($iAnzahl) {
			$this->iAnzahlZeilenBlind = $iAnzahl;
		}
		public function getAnzahlSpalten() {
			return count ( $this->aSpalten );
		}
		
		/**
		 * Gibt die Anzahl aller Button-Spalten zurck
		 *
		 * @return number
		 */
		protected function getAnzahlButtons() {
			$i = 0;
			
			foreach ( $this->aSpalten as $oSpalte ) {
				if (get_class ( $oSpalte ) == "ListenspalteButton") {
					$i ++;
				}
			}
			
			return $i;
		}
		
		/**
		 * Prft, ob das angegebene Objekt vom Typ ListenspalteButton ist.
		 *
		 * @param unknown_type $oSpalte        	
		 */
		protected function isButtonSpalte($oSpalte) {
			if (get_class ( $oSpalte ) == "ListenspalteButton") {
				return true;
			}
			
			return false;
		}
		
		/**
		 * Gibt eine Referenz zum Spaltenobjekt anhand des Feldnamens
		 *
		 * @param unknown_type $sFeldname        	
		 */
		protected function getSpalteByFeldname($sFeldname) {
			foreach ( $this->aSpalten as &$oSpalte ) {
				if ($oSpalte->sFeldname == $sFeldname) {
					return $oSpalte;
				}
			}
			
			return null;
		}
		
		/**
		 * Gibt die Summe der Breite der Buttonspalten aus.
		 *
		 * @return number
		 */
		protected function getBreiteVonButtons() {
			$iBreite = 0;
			
			foreach ( $this->aSpalten as $oSpalte ) {
				if ($this->isButtonSpalte ( $oSpalte )) {
					$iBreite += $oSpalte->iBreite;
				}
			}
			
			return $iBreite;
		}
		
		/**
		 * Gibt die Breite aller Spalten aus
		 */
		protected function getBreite() {
			$iBreite = 0;
			foreach ( $this->aSpalten as $oSpalte ) {
				$iBreite += $oSpalte->iBreite;
			}
			
			return $iBreite;
		}
		
		/**
		 * Bestimmt, ob eine Spalte rechtsbndig ausgerichtet werden muss
		 */
		protected function bestimmeAusrichtung() {
			if ($this->bAusrichtungBestimmt) {
				return;
			}
			
			$aOutputZeile = $this->aZeilen [0];
			
			if (! is_array ( $aOutputZeile )) {
				return;
			}
			
			foreach ( $aOutputZeile as $sFeldname => $sFeldinhalt ) {
				$oSpalte = $this->getSpalteByFeldname ( $sFeldname );
				
				if ($oSpalte->bAusrichtungRechts) {
					continue;
				}
				
				if ($oSpalte->bAusrichtungFix) {
					continue;
				}
				
				$sFeldinhalt = trim ( $sFeldinhalt );
				
				if (is_numeric ( str_replace ( array (
						" ",
						",",
						":",
						"&euro;",
						"%" 
				), "", $sFeldinhalt ) )) {
					$oSpalte->bAusrichtungRechts = true;
				}
			}
			
			$this->bAusrichtungBestimmt = true;
		}
		
		/**
		 * Spaltenbreiten aus Datenbank berschreiben
		 */
		protected function ladeSpaltenbreitenAusDatenbank() {
			$aUserRow = holeBenutzer ();
			
			if (! empty ( $aUserRow ['dtArraySpaltenbreiten'] )) {
				$aSpaltenbreiten = unserialize ( $aUserRow ['dtArraySpaltenbreiten'] );
			}
			
			if (! is_array ( $aSpaltenbreiten [$this->sListeName] )) {
				return;
			}
			
			foreach ( $aSpaltenbreiten [$this->sListeName] as $sSpalteName => $iSpalteBreite ) {
				$oSpalte = $this->getSpalteByFeldname ( $sSpalteName );
				$oSpalte->iBreite = $iSpalteBreite;
			}
		}
		
		/**
		 * Versucht lesbaren Text aus Formularen zu machen
		 */
		protected function getPrintableText($sText) {
			$aTextParts = explode ( "<input", $sText );
			$sTextNeu = "";
			
			foreach ( $aTextParts as $sPart ) {
				if (strpos ( $sPart, "type=\"checkbox\"" ) !== false) {
					if (strpos ( $sPart, "checked" ) !== false) {
						$sPart = "X";
					} else {
						$sPart = "O";
					}
				} elseif (strpos ( $sPart, "type=\"text\"" ) !== false) {
					$sPartOrig = $sPart;
					$sPart = "";
					
					/*
					 * Datumsfeld KW
					 */
					if (strpos ( $sPartOrig, "_kw\"" ) !== false) {
						$sPart .= " (KW ";
					}
					
					$sPart .= $this->getText ( $sPartOrig );
					
					/*
					 * Datumsfelder Tagname, Tag, Monat, Jahr
					 */
					if (strpos ( $sPartOrig, "_tagname\"" ) !== false) {
						$sPart .= " ";
					} elseif (strpos ( $sPartOrig, "_tag\"" ) !== false || strpos ( $sPartOrig, "_monat\"" ) !== false) {
						$sPart .= ".";
					} elseif (strpos ( $sPartOrig, "_kw\"" ) !== false) {
						$sPart .= ")";
					}
				} elseif (strpos ( $sPart, "type=\"hidden\"" ) !== false) {
					$sPartOrig = $sPart;
					$sPart = "";
				} elseif (strpos ( $sPart, "<textarea" ) !== false) {
					$iTextareaStart = strpos ( $sPart, "<textarea" );
					$sPart = substr ( $sPart, $iTextareaStart );
					$sPart = substr ( $sPart, strpos ( $sPart, ">" ) + 1 );
					$sPart = substr ( $sPart, 0, strpos ( $sPart, "</textarea" ) );
				}
				
				$sPart = html_entity_decode ( $sPart );
				$sPart = str_replace ( "<br>", " ", $sPart );
				$sPart = strip_tags ( $sPart );
				$sTextNeu .= $sPart;
			}
			
			return $sTextNeu;
		}
		
		/**
		 * Textvalue aus Input Feldern ziehen
		 *
		 * @param string $sText        	
		 */
		protected function getText($sText) {
			$sSuchstringValue = " value="; // Leerzeichen vorne, um wirklich den HTML-Tag zu bekommen
			$aTextNeu = array ();
			
			while ( substr_count ( $sText, $sSuchstringValue ) > 0 ) {
				$sTrennzeichen = substr ( $sText, strpos ( $sText, $sSuchstringValue ) + strlen ( $sSuchstringValue ), 1 ); // Normalerweise ' oder "
				
				$iValueInhaltStart = strpos ( $sText, $sSuchstringValue ) + strlen ( $sSuchstringValue . $sTrennzeichen );
				$sText = substr ( $sText, $iValueInhaltStart ); // Text nach value=", bis Ende
				$iValueInhaltEnde = strpos ( $sText, $sTrennzeichen );
				
				$aTextNeu [] = substr ( $sText, 0, $iValueInhaltEnde ); // Text innerhalb von value="", in Array schreiben
				$sText = substr ( $sText, $iValueInhaltEnde );
			}
			
			return join ( ", ", $aTextNeu ); // Array kommagetrennt Returnen
		}
	}
}

if (! class_exists ( "ListeHTML" )) {
	class ListeHTML extends Liste {
		protected $iGesamtbreiteKopf = 0;
		public $bMaximiereHoehe = true;
		public $bShowOptionButton = true;
		public $bShowListenfuss = true;
		
		/**
		 * Ersetzt bestimmte Styleklassen in einem String
		 *
		 * @param string $sText        	
		 */
		protected function ersetzeStyleKlassen($sText) {
			$aKlassenErsetzungVon = array (
					"class=\"zuruecksetzenbutton\"",
					"class=\"aenderungszeiger\"",
					"class=\"klickbutton\"" 
			);
			$aKlassenErsetzungIn = array (
					"class=\"zuruecksetzenbutton_liste\"",
					"class=\"aenderungszeiger_liste\"",
					"class=\"klickbutton_liste\"" 
			);
			
			return str_replace ( $aKlassenErsetzungVon, $aKlassenErsetzungIn, $sText );
		}
		
		/**
		 * Ldt die Option "maximiere Hhe" aus der Datenbank
		 */
		protected function ladeHoeheMaximierenOptionAusDatenbank() {
			$aUserRow = holeBenutzer ();
			
			if (! empty ( $aUserRow ['dtArrayListenoptionen'] )) {
				$aListenoptionen = unserialize ( $aUserRow ['dtArrayListenoptionen'] );
			}
			
			if (! isset ( $aListenoptionen [$this->sListeName] ["bMaximiereHoehe"] )) {
				return;
			}
			
			$this->bMaximiereHoehe = $aListenoptionen [$this->sListeName] ["bMaximiereHoehe"];
		}
		
		/**
		 * Prft, ob ein Quicklinkbutton vorhanden ist, schneidet ihn ggf.
		 * ab und platziert ihn im Inputfeld
		 *
		 * @param
		 *        	string Spalte
		 * @return string Spalte
		 */
		protected function schneideButtonAb($sText) {
			$sTextStart = "<!--Button-->";
			$sTextEnde = "<!--ButtonEnde-->";
			
			if (substr_count ( $sText, $sTextStart ) != 1) {
				return $sText;
			}
			
			$sDivId = uniqid ();
			$sOuterDiv = "<div style='position: relative;' onMouseOver='$(\"#" . $sDivId . "\").show();' onMouseOut='$(\"#" . $sDivId . "\").hide();'>";
			$sInnerDiv = "<div id='" . $sDivId . "' class='liste_inhalt_quicklinkbutton'>";
			
			$sText = str_replace ( $sTextStart, $sInnerDiv, $sText );
			$sText = str_replace ( $sTextEnde, "</div>", $sText );
			$sText = $sOuterDiv . $sText . "</div>";
			
			return $sText;
		}
		
		/**
		 * Prft, wie viele Input-Felder im Text vorhanden sind.
		 * Bei einem Inputfeld wird dessen Gre nach Spaltenbreite angepasst.
		 *
		 * @param string $sText        	
		 * @return string
		 */
		protected function bestimmeInputFeldGroesse($sText, $iSpaltenbreite) {
			$iSpaltenbreite = $iSpaltenbreite - 3;
			
			$sSuchtextInput = "<input";
			$sSuchtextStyle = " style=";
			
			if ($this->getAnzahlInputFields ( $sText ) != 1) {
				return $sText;
			}
			
			$iStrPosInput = stripos ( $sText, $sSuchtextInput ); // Nur innerhalb des Input Felds arbyten
			$iStrPosInputEnde = strpos ( $sText, ">", $iStrPosInput );
			
			if (($iStrPosStyleAnfang = stripos ( $sText, $sSuchtextStyle, $iStrPosInput )) !== false && $iStrPosStyleAnfang < $iStrPosInputEnde) {
				$iStrPosStyleAnfang += strlen ( $sSuchtextStyle ) + 1; // Position nach style="
				$iStrPosStyleEnde = strpos ( $sText, "\"", $iStrPosStyleAnfang );
				
				$sStyleTag = substr ( $sText, $iStrPosStyleAnfang, $iStrPosStyleEnde - $iStrPosStyleAnfang );
				$sStyleTag .= " width:" . $iSpaltenbreite . "px;";
				
				$sTextNeu = substr ( $sText, 0, $iStrPosStyleAnfang );
				$sTextNeu .= $sStyleTag;
				$sTextNeu .= substr ( $sText, $iStrPosStyleEnde );
				return $sTextNeu;
			} else {
				$iStrPosInputEnde = strpos ( $sText, ">", $iStrPosInput );
				$sTextNeu = substr ( $sText, 0, $iStrPosInputEnde - 1 ) . " style=\"width: " . $iSpaltenbreite . "px;\"" . substr ( $sText, $iStrPosInputEnde );
				return $sTextNeu;
			}
		}
		
		/**
		 * Gibt die Anzahl der im Text vorhandenen Input-Felder zurck
		 *
		 * @param int $sText        	
		 * @return number
		 */
		protected function getAnzahlInputFields($sText) {
			$sSuchtext = array ();
			$sSuchtext [] = "<input";
			$sSuchtext [] = "<button";
			$iErgebnisse = 0;
			
			foreach ( $sSuchtext as $sStr ) {
				$iErgebnisse += substr_count ( $sText, $sStr );
			}
			
			return $iErgebnisse;
		}
		
		/**
		 * Listenkopf ausgeben
		 */
		protected function getListenkopf() {
			global $zeilenhintergrund_listenkopf, $bgcolor_formular;
			global $target, $typ, $id, $subtarget, $subtyp, $subid;
			global $sortierung;
			
			$aSortierung = $sortierung [$this->sListeName];
			$iAnzahlButtonSpalten = 0;
			
			$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
			
			$html = "<table border='0' cellspacing='0' cellpadding='0' class='text' style='background-color: " . $bgcolor_formular . ";' width='100%'>";
			$html .= "<tr><td class='formularrahmen_oben'></td></tr>";
			$html .= "<tr><td>";
			
			$html .= "<table border='0' cellspacing='0' cellpadding='1' class='text'>";
			$html .= "<tr style='background-color: " . $zeilenhintergrund_listenkopf . ";'>";
			
			for($i = 0; $i < $this->getAnzahlSpalten (); $i ++) {
				$oSpalte = $this->aSpalten [$i];
				
				// Keine leeren Titel
				if (empty ( $oSpalte->sTitel )) {
					$oSpalte->sTitel = "&nbsp;";
				}
				
				$iKopfSpaltenbreite = $oSpalte->iBreite - 2;
				
				if ($oSpalte->iBreite < 1) {
					$html .= "<td></td>";
				} elseif (! $oSpalte->bNichtSortieren) {
					if ($aSortierung ["feldname"] == $oSpalte->sSortierFeld) {
						if (empty ( $aSortierung ["richtung"] )) {
							$sRichtung = "desc";
							$sRichtungZeichen = "&darr;&darr;&darr;";
							$sRichtungName = "aufsteigend";
						} else {
							$sRichtung = "";
							$sRichtungZeichen = "&uarr;&uarr;&uarr;";
							$sRichtungName = "absteigend";
						}
					} elseif ($aSortierung ["feldname2"] == $oSpalte->sSortierFeld) {
						if (empty ( $aSortierung ["richtung2"] )) {
							$sRichtung = "desc";
							$sRichtungZeichen = "&darr;&darr;";
							$sRichtungName = "aufsteigend";
						} else {
							$sRichtung = "";
							$sRichtungZeichen = "&uarr;&uarr;";
							$sRichtungName = "absteigend";
						}
					} elseif ($aSortierung ["feldname3"] == $oSpalte->sSortierFeld) {
						if (empty ( $aSortierung ["richtung3"] )) {
							$sRichtung = "desc";
							$sRichtungZeichen = "&darr;";
							$sRichtungName = "aufsteigend";
						} else {
							$sRichtung = "";
							$sRichtungZeichen = "&uarr;";
							$sRichtungName = "absteigend";
						}
					} else {
						$sRichtung = "";
						$sRichtungZeichen = "";
						$sRichtungName = "absteigend";
					}
					
					$this->iGesamtbreiteKopf += $iKopfSpaltenbreite;
					
					$html .= "<td class='liste_kopf_spalte'><div class='liste_kopf_spalte_resizable' style='width: " . $iKopfSpaltenbreite . "px;' listenname='" . $this->sListeName . "' name='" . $oSpalte->sFeldname . "'>";
					
					$html .= "<a class='liste_kopf_sortierlink' listenname='" . $this->sListeName . "' href='main.php?" . $ziel;
					$html .= "&sortierung_name=" . $this->sListeName . "&sortierung_feld=" . $oSpalte->sSortierFeld . "' ";
					$html .= "title='nach diesem Feld sortieren'>" . $oSpalte->sTitel . "</a>";
					
					$html .= "<a class='liste_kopf_sortierlink' listenname='" . $this->sListeName . "' href='main.php?" . $ziel;
					$html .= "&sortierung_name=" . $this->sListeName . "&sortierung_feld=" . $oSpalte->sSortierFeld . "&sortierung_richtung=" . $sRichtung . "' ";
					$html .= "title='nach diesem Feld " . $sRichtungName . " sortieren'>" . $sRichtungZeichen . "</a>";
					$html .= "</div></td>";
				} elseif ($this->isButtonSpalte ( $oSpalte )) {
					$iAnzahlButtonSpalten ++;
					
					if ($iAnzahlButtonSpalten != 1) {
						continue;
					}
					
					$this->iGesamtbreiteKopf += $this->getBreiteVonButtons ();
					
					/*
					 * Buttons
					 */
					$html .= "<td class='liste_kopf_spalte_button'><div listenname='" . $this->sListeName . "' name='buttons' style='width: " . $this->getBreiteVonButtons () . "px; position: relative;'>";
					
					if ($this->bShowOptionButton) {
						$html .= $this->getSubmenu ();
						$html .= "<a listenname='" . $this->sListeName . "' class='liste_kopf_togglemenu' href='#' title='Men ffnen'><img align='texttop' style='border: 0px;' src='config/pics/pulldown_menubutton.png'> Optionen</a>&nbsp;&nbsp;";
					}
					
					$html .= "</div>";
					$html .= "</td>";
				} else {
					$this->iGesamtbreiteKopf += $iKopfSpaltenbreite;
					
					$html .= "<td class='liste_kopf_spalte'><div class='liste_kopf_spalte_resizable' style='width: " . $iKopfSpaltenbreite . "px;' listenname='" . $this->sListeName . "' name='" . $oSpalte->sFeldname . "'>";
					$html .= $oSpalte->sTitel;
					$html .= "</div></td>";
				}
			}
			
			$html .= "</tr>";
			$html .= "</table>";
			
			$html .= "</td></tr>";
			$html .= "</table>";
			
			return $html;
		}
		
		/**
		 * Listeninhalt ausgeben
		 */
		protected function getListenkoerper() {
			global $target, $typ, $id, $subtarget, $subtyp, $subid;
			global $ajaxtarget, $zeilenhintergrund1, $zeilenhintergrund2, $zeilenzahl;
			
			$html = "";
			$iZeilenzahl = 0;
			
			if (! isset ( $ajaxtarget )) {
				if ($this->bMaximiereHoehe or $this->bManuelleHoehe != 0) {
					$sClasses = "liste_koerper liste_koerper_height";
				} else {
					$sClasses = "liste_koerper";
				}
				
				if ($this->bManuelleHoehe != 0) {
					$sStyle = "style=\"height: " . $this->bManuelleHoehe . "px;\"";
				}
				
				$html .= "<div class='" . $sClasses . "' $sStyle>";
			}
			
			for($i = 0; $i < $this->getAnzahlZeilen (); $i ++) {
				$aZeile = &$this->aZeilen [$i];
				
				if ($aZeile ['Textfilter'] == "nomatch") {
					continue;
				}
				
				/*
				 * Namen des DIV oder TR bestimmen
				 */
				
				if (isset ( $aZeile ['id'] ) && substr ( $aZeile ['id'], 0, 4 ) == "<!--") {
					$iId = ( int ) str_replace ( array (
							"<!--",
							"//-->" 
					), "", $aZeile ['id'] );
					
					$sIdString = "id=\"row_id_" . $iId . "\"";
				}
				
				/*
				 * Zeilen zhlen?
				 */
				if (! isset ( $ajaxtarget )) {
					$iZeilenzahl ++;
				} else {
					$iZeilenzahl = $zeilenzahl;
				}
				
				/*
				 * Hindergrundfarbe wechseln
				 */
				if ($iZeilenzahl % 2) {
					$sStyleZeileHintergrund = $zeilenhintergrund1;
				} else {
					$sStyleZeileHintergrund = $zeilenhintergrund2;
				}
				
				/*
				 * Formularnamen setzen
				 */
				if ($this->getAnzahlZeilen () == 1) {
					$sFormName = "ajaxlisteform" . uniqid ();
				} else {
					$sFormName = "ajaxlisteform" . $i;
				}
				
				/*
				 * Formular wenn kein Ajax
				 */
				if (! isset ( $ajaxtarget )) {
					if ($typ != "fenster-dtausexport")
						$html .= "<form name='" . $sFormName . "' action='" . $_SERVER ['PHP_SELF'] . "' enctype='multipart/form-data' method='post' style='display: inline;'>";
					
					$html .= "<input type='hidden' name='zeilenzahl' value='" . $iZeilenzahl . "'>";
					$html .= "<div " . $sIdString . ">";
				}
				
				$html .= "<table border='0' cellspacing='0' cellpadding='0' class='text' style='table-layout: fixed;' id=\"table_" . $this->sListeName . "\">";
				$html .= "<tr style='background-color: " . $sStyleZeileHintergrund . ";'>";
				
				/*
				 * Durchlaufe alle Spalten
				 */
				for($n = 0; $n < $this->getAnzahlSpalten (); $n ++) {
					$oSpalte = $this->aSpalten [$n];
					
					/*
					 * Ausrichtung
					 */
					if ($oSpalte->bAusrichtungRechts) {
						$sStyleTextAlign = "text-align: right;";
						$sAbstand = "&nbsp;&nbsp;";
					} else {
						$sStyleTextAlign = "";
						$sAbstand = "";
					}
					
					/*
					 * Spalteninhalt bearbeiten
					 */
					$aZeile [$oSpalte->sFeldname] = $this->ersetzeStyleKlassen ( $aZeile [$oSpalte->sFeldname] );
					
					if ($this->getAnzahlInputFields ( $aZeile [$oSpalte->sFeldname] ) == 1 && ! $this->isButtonSpalte ( $oSpalte )) {
						$aZeile [$oSpalte->sFeldname] = $this->schneideButtonAb ( $aZeile [$oSpalte->sFeldname] );
						$aZeile [$oSpalte->sFeldname] = $this->bestimmeInputFeldGroesse ( $aZeile [$oSpalte->sFeldname], $oSpalte->iBreite );
					}
					
					if ($this->getAnzahlInputFields ( $aZeile [$oSpalte->sFeldname] ) == 0) {
						$html .= "<td style='width: " . $oSpalte->iBreite . "px;' class='liste_koerper_spalte'>";
						
						$html .= "<div style='width: " . $oSpalte->iBreite . "px; " . $sStyleTextAlign . "' class='liste_koerper_spalte_div'>";
						$html .= $aZeile [$oSpalte->sFeldname] . $sAbstand;
						$html .= "</div>";
						
						$html .= "</td>";
					} else {
						if ($aZeile ["dtVarianteNr"])
							$html .= "<td style='min-width: " . $oSpalte->iBreite . "px;' class='liste_koerper_spalte liste_koerper_moz'>";
						
						else
							$html .= "<td style='width: " . $oSpalte->iBreite . "px;' class='liste_koerper_spalte liste_koerper_moz'>";
						
						$html .= $aZeile [$oSpalte->sFeldname] . $sAbstand;
						
						$html .= "</td>";
					}
				}
				
				$html .= "</tr></table>";
				
				if (! isset ( $ajaxtarget )) {
					$html .= "</div>";
					
					if ($typ != "fenster-dtausexport")
						$html .= "</form>";
				}
			}
			
			if (! isset ( $ajaxtarget )) {
				$html .= "</div>";
			}
			
			return $html;
		}
		
		/**
		 * Listenfuss ausgeben
		 *
		 * @param int $iAnzahlAusgabe        	
		 * @param int $iAnzahlGesamt        	
		 * @return string
		 */
		protected function getListenfuss($iAnzahlAusgabe, $iAnzahlGesamt) {
			global $target, $typ, $id, $subtarget, $subtyp, $subid;
			global $sortierung;
			global $zeilenhintergrund_listenkopf, $bgcolor_formular;
			
			$aSortierung = $sortierung [$this->sListeName];
			$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
			
			/*
			 * Textausgabe
			 */
			$sAnzahlText = "Auswahl: " . $iAnzahlAusgabe . " von " . $iAnzahlGesamt;
			
			if ($aSortierung ["anzahl"] != "") {
				$iAnzeigeGesamt = $this->getAnzahlZeilenAusgabe ();
				
				$iPositionEnde = (min ( $iAnzahlAusgabe, ($aSortierung ["position"] + $aSortierung ["anzahl"] - 1) ));
				$sAnzahlText .= "&nbsp;&nbsp;|&nbsp;&nbsp;Anzeige: " . $iAnzeigeGesamt . " Eintr&auml;ge";
				if ($iPositionEnde > 0) {
					$sAnzahlText .= "&nbsp;&nbsp;|&nbsp;&nbsp;Position von " . $aSortierung ["position"] . " bis " . $iPositionEnde;
				}
			}
			
			$html = "<table border='0' cellspacing='0' cellpadding='0' class='text' style='background-color: " . $bgcolor_formular . ";' width='100%'>";
			$html .= "<tr><td>";
			
			$html .= "<table border='0' cellspacing='0' cellpadding='1' class='text'>";
			$html .= "<tr style='background-color: " . $zeilenhintergrund_listenkopf . ";'>";
			
			$html .= "<td style='width: " . ($this->iGesamtbreiteKopf + 30) . "px;'><div class='liste_fuss_div_links'>" . $sAnzahlText . "</div><div class='liste_fuss_div_rechts'>";
			$html .= $this->getListenfussPageButtons ( $aSortierung ["anzahl"], $aSortierung ["position"], $iAnzahlAusgabe );
			$html .= $this->getListenfussAnzeigebegrenzer ( $aSortierung ["anzahl"], $aSortierung ["position"] );
			$html .= "</div></td>";
			$html .= "</tr>";
			$html .= "</table>";
			
			$html .= "</td></tr>";
			$html .= "<tr><td class='formularrahmen_unten'></td></tr>";
			
			$html .= "</table>";
			
			return $html;
		}
		
		/**
		 * Pagebuttons in der Fusszeile
		 *
		 * @param int $iAktuelleLimitierung        	
		 * @param int $iPosition        	
		 * @param int $iAnzahlAusgabe        	
		 * @return string
		 */
		protected function getListenfussPageButtons($iAktuelleLimitierung, $iPosition, $iAnzahlAusgabe) {
			global $target, $typ, $id, $subtarget, $subtyp, $subid;
			global $zeilenhintergrund_listenkopf;
			$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
			
			$html = "";
			
			if (! empty ( $iAktuelleLimitierung )) {
				if ($iPosition != 1) {
					/*
					 * Springe zu Position 1
					 */
					$iPositionVon = 1;
					$iPositionBis = min ( $iAnzahlAusgabe, ($iPositionVon + $iAktuelleLimitierung - 1) );
					
					$html .= "<a class='pagingbuttons' listenname='" . $this->sListeName . "' href='main.php?" . $ziel . "&sortierung_name=" . $this->sListeName . "&sortierung_position=" . $iPositionVon . "' ";
					$html .= "title='Springe zum Anfang; Zeige " . $iPositionVon . " bis " . $iPositionBis . "'>";
					$html .= "<b>&lt;&lt;</b></a>&nbsp;&nbsp;";
					
					/*
					 * Springe zu Position - Anzahl
					 */
					$iPositionVon = max ( 1, $iPosition - $iAktuelleLimitierung );
					$iPositionBis = min ( $iAnzahlAusgabe, ($iPositionVon + $iAktuelleLimitierung - 1) );
					
					$html .= "<a class='pagingbuttons' listenname='" . $this->sListeName . "' href='main.php?" . $ziel . "&sortierung_name=" . $this->sListeName . "&sortierung_position=" . $iPositionVon . "' ";
					$html .= "title='Zeige " . $iPositionVon . " bis " . $iPositionBis . "'>";
					$html .= "<b>&lt;</b></a>";
				} else {
					$html .= "<font color='" . $zeilenhintergrund_listenkopf . "'><b>&lt;&lt;</b></a>&nbsp;&nbsp;<b>&lt;</b></font>";
				}
				
				$html .= "&nbsp;&nbsp;|&nbsp;&nbsp;";
				
				if ($iPosition <= ($iAnzahlAusgabe - $iAktuelleLimitierung)) {
					
					/*
					 * Springe zu Position + Anzahl
					 */
					$iPositionVon = max ( 1, min ( ($iAnzahlAusgabe - $iAktuelleLimitierung + 1), ($iPosition + $iAktuelleLimitierung) ) );
					$iPositionBis = min ( $iAnzahlAusgabe, ($iPositionVon + $iAktuelleLimitierung - 1) );
					
					$html .= "<a class='pagingbuttons' listenname='" . $this->sListeName . "' href='main.php?" . $ziel . "&sortierung_name=" . $this->sListeName . "&sortierung_position=" . $iPositionVon . "' ";
					$html .= "title='Zeige " . $iPositionVon . " bis " . $iPositionBis . "'>";
					$html .= "<b>&gt;</b></a>&nbsp;&nbsp;";
					
					/*
					 * Springe zur letzten Position
					 */
					$iPositionVon = max ( 1, $iAnzahlAusgabe - $iAktuelleLimitierung + 1 );
					$iPositionBis = min ( $iAnzahlAusgabe, ($iPositionVon + $iAktuelleLimitierung - 1) );
					
					$html .= "<a class='pagingbuttons' listenname='" . $this->sListeName . "' href='main.php?" . $ziel . "&sortierung_name=" . $this->sListeName . "&sortierung_position=" . $iPositionVon . "' ";
					$html .= "title='Sprige zum Ende; Zeige " . $iPositionVon . " bis " . $iPositionBis . "'>";
					$html .= "<b>&gt;&gt;</b></a>&nbsp;&nbsp;";
				} else {
					$html .= "<font color='" . $zeilenhintergrund_listenkopf . "'><b>&gt;</b></a>&nbsp;&nbsp;<b>&gt;&gt;</b></a>&nbsp;&nbsp;</font>";
				}
			}
			
			return $html;
		}
		
		/**
		 * Dropdownbox zur Auswahl der Listenlnge
		 *
		 * @param int $iAktuelleLimitierung        	
		 * @param int $iPosition        	
		 * @return string
		 */
		protected function getListenfussAnzeigebegrenzer($iAktuelleLimitierung, $iPosition) {
			global $target, $typ, $id, $subtarget, $subtyp, $subid;
			$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
			
			$html = "<select listenname='" . $this->sListeName . "' size='1' class='input_select_50 anzeigebegrenzung' title='Anzeige begrenzen auf ... Eintrge'>";
			
			$aSelectbox = array (
					"alle" => "",
					"500" => "500",
					"100" => "100",
					"50" => "50",
					"20" => "20",
					"10" => "10" 
			);
			
			foreach ( $aSelectbox as $sOptionName => $sOptionValue ) {
				
				if (($iPosition + $sOptionValue) > $iAktuelleLimitierung) {
					$iPositionNeu = max ( 1, $iAktuelleLimitierung - $sOptionValue + 1 );
				} else {
					$iPositionNeu = $iPosition;
				}
				
				if ($iAktuelleLimitierung == $sOptionValue) {
					$sSelected = "selected";
				} else {
					$sSelected = "";
				}
				
				$html .= "<option value='" . $sOptionValue . "/" . $iPositionNeu . "' " . $sSelected . ">" . $sOptionName . "</option>";
			}
			$html .= "</select>";
			
			return $html;
		}
		
		/**
		 * Submen ausgeben
		 */
		protected function getSubmenu() {
			$html .= "<div class='liste_submenu'>";
			$html .= "<table class='text' style='background-color: #D6E1F1;'>";
			
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_savesortierung' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_sortierungsave.png'></td><td>Sortierung speichern</td></tr>";
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_resetsortierung' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_sortierungreset.png'></td><td>Sortierung zurcksetzen</td></tr>";
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_resetspaltenbreite' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_spaltenreset.png'></td><td>Spaltenbreite zurcksetzen</td></tr>";
			
			$html .= "<tr><td colspan='2'><hr></td></tr>";
			
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_outputcsv' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_csvexport.png'></td><td>CSV Export</td></tr>";
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_druckhoch' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_drucken_hochformat.png'></td><td>Drucken Hochformat</td></tr>";
			$html .= "<tr class='liste_submenu_menupunkt liste_submenu_druckquer' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_drucken_querformat.png'></td><td>Drucken Querformat</td></tr>";
			
			if ($this->bEigenesDrucken) {
				$html .= "<tr class='liste_submenu_menupunkt liste_submenu_eigenesdrucken' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_drucken.png'></td><td>Drucken</td></tr>";
			}
			
			$html .= "<tr><td colspan='2'><hr></td></tr>";
			
			if ($this->bMaximiereHoehe) {
				$html .= "<tr class='liste_submenu_menupunkt liste_submenu_hoeheliste' listenname='" . $this->sListeName . "'><td><img src='config/pics/pulldown_liste_inhalt.png'></td><td>Listenhhe an Inhalt anpassen</td></tr>";
			} else {
				$html .= "<tr class='liste_submenu_menupunkt liste_submenu_hoehebrowser' listenname='" . $this->sListeName . "' $sStyle><td><img src='config/pics/pulldown_liste_browser.png'></td><td>Listenhhe an Browser anpassen</td></tr>";
			}
			
			$html .= "</table></div>";
			
			return $html;
		}
		
		/**
		 * Fgt bei Bedarf eine ButtonSpalte hinzu.
		 */
		protected function checkButtonSpalte() {
			if ($this->getAnzahlButtons () == 0) {
				$this->addSpalte ( new ListenspalteButton ( "Buttons", 80 ) );
			}
		}
		
		/**
		 * Erzeugt DIV-Box fr CSV Output
		 */
		protected function getCSVDiv() {
			$sAusgabe = "<div class='liste_csv_output_div'><textarea class='liste_csv_output_textarea'></textarea><p style='text-align: center;'>" . klickbutton ( "schliessen", "onclick=\"$('.liste_csv_output_div').toggle();\"" ) . "</p></div>";
			return $sAusgabe;
		}
		
		/**
		 * Liste ausgeben
		 *
		 * @param
		 *        	int Anzahl aller anzuzeigenden Elemente (nach Filterung)
		 * @param
		 *        	int Anzahl aller anzeigbaren Elemente (vor Filterung)
		 */
		public function output($iAnzahlAusgabe, $iAnzahlGesamt, $iManuelleHoehe = "") {
			global $ajaxtarget, $typ, $subtyp;
			
			$this->bestimmeAusrichtung ();
			$this->ladeSpaltenbreitenAusDatenbank ();
			
			if ($this->bShowOptionButton) {
				$this->checkButtonSpalte ();
			}
			
			$this->ladeHoeheMaximierenOptionAusDatenbank ();
			
			if ($iManuelleHoehe != "") {
				$this->bMaximiereHoehe = false;
				$this->bManuelleHoehe = $iManuelleHoehe;
			}
			if (! $ajaxtarget) {
				/*
				 * CSV Div
				 */
				echo $this->getCSVDiv ();
				if (! $_GET ['ajaxliste']) {
					echo "<div id='listenkopf-" . $this->sListeName . "'>";
				}
				
				/*
				 * Listenkopf
				 */
				echo "<!--Listenkopf:" . $this->sListeName . "-->";
				
				if ($typ == "list") {
					$jsvar = "var typ=\"list\"; ";
				} elseif ($subtyp == "list") {
					$jsvar = "var subtyp=\"list\"; ";
				}
				
				if ($this->bMaximiereHoehe) {
					echo "<script>" . $jsvar . " var bListeHoeheMaximum = true;</script>";
				} else {
					echo "<script>" . $jsvar . " var bListeHoeheMaximum = false;";
					
					if ($this->bManuelleHoehe != 0) {
						echo "var bManuelleHoehe = " . $this->bManuelleHoehe . "; ";
					}
					
					echo "</script>";
				}
				
				echo $this->getListenkopf ();
				
				echo "<!--ListenkopfEnde:" . $this->sListeName . "-->";
				
				if (! $_GET ['ajaxliste']) {
					echo "</div>";
				}
			}
			
			if (! $ajaxtarget) {
				if (! $_GET ['ajaxliste']) {
					echo "<div id='liste-" . $this->sListeName . "'>";
				}
				
				echo "<!--Liste:" . $this->sListeName . "-->";
			}
			
			/*
			 * Listenkrper
			 */
			echo $this->getListenkoerper ();
			
			/*
			 * Listenfuss
			 */
			if (! $ajaxtarget && $this->bShowListenfuss) {
				if ($typ != "fenster-dtausexport")
					echo $this->getListenfuss ( $iAnzahlAusgabe, $iAnzahlGesamt );
				
				echo "<!--ListeEnde:" . $this->sListeName . "-->";
				
				if (! $_GET ['ajaxliste']) {
					echo "</div>";
				}
			}
		}
	}
}

if (! class_exists ( "ListeCSV" )) {
	class ListeCSV extends Liste {
		protected $sTrennzeichen = "\t";
		protected $sZeilenumbruch = "\r\n";
		
		/**
		 * Seperater Konstruktor um Listenlimitierung zu berschreiben.
		 *
		 * @param string $sListeName        	
		 * @param bool $bEigenesDrucken
		 *        	- Wenn true wird das Target drucken.php verwendet
		 */
		public function __construct($sListeName, $bEigenesDrucken = false) {
			global $sortierung;
			
			$sortierung [$this->sListeName] ["anzahl"] = "";
			parent::__construct ( $sListeName, $bEigenesDrucken );
		}
		
		/**
		 * Listenkopf
		 */
		protected function getListenkopf() {
			$sAusgabe = "";
			
			for($i = 0; $i < $this->getAnzahlSpalten (); $i ++) {
				$oSpalte = $this->aSpalten [$i];
				
				if ($oSpalte->iBreite < 1 || $this->isButtonSpalte ( $oSpalte )) {
					continue;
				}
				
				if (empty ( $oSpalte->sTitel )) {
					$oSpalte->sTitel = "kein Titel";
				}
				
				$sAusgabe .= $this->replaceEuroSign ( $oSpalte->sTitel ) . $this->sTrennzeichen;
			}
			
			$sAusgabe = trim ( $sAusgabe );
			
			return $sAusgabe;
		}
		protected function replaceEuroSign($sText) {
			Global $sCSVExportWaehrung;
			
			return str_replace ( "&euro;", $sCSVExportWaehrung, $sText );
		}
		
		/**
		 * Listenkrper
		 */
		protected function getListenkoerper() {
			$sAusgabe = array ();
			
			for($i = 0; $i < $this->getAnzahlZeilen (); $i ++) {
				$aZeile = &$this->aZeilen [$i];
				$sZeile = "";
				
				/*
				 * Durchlaufe alle Spalten
				 */
				for($n = 0; $n < $this->getAnzahlSpalten (); $n ++) {
					$oSpalte = $this->aSpalten [$n];
					
					if ($oSpalte->iBreite < 1 || $this->isButtonSpalte ( $oSpalte )) {
						continue;
					}
					
					/*
					 * Spalteninhalt bearbeiten
					 */
					$aZeile [$oSpalte->sFeldname] = $this->getPrintableText ( $aZeile [$oSpalte->sFeldname] );
					$aZeile [$oSpalte->sFeldname] = $this->replaceEuroSign ( $aZeile [$oSpalte->sFeldname] );
					$aZeile [$oSpalte->sFeldname] = str_replace(array("\r\n","\n","\r"),"",$aZeile [$oSpalte->sFeldname]);
					
					
					$sZeile .= $aZeile [$oSpalte->sFeldname] . $this->sTrennzeichen;
				}
				
				$sAusgabe [] = trim ( $sZeile, " " );
			}
			
			return join ( $this->sZeilenumbruch, $sAusgabe );
		}
		public function output() {
			echo "<!--CSVOutputStart-->";
			echo $this->getListenkopf () . $this->sZeilenumbruch;
			echo $this->getListenkoerper ();
			echo "<!--CSVOutputEnde-->";
		}
	}
}

if (! class_exists ( "ListePDF" )) {
	class ListePDF extends Liste {
		protected $bQuerformat = false;
		protected $iZeilenSpacing = 0.1;
		protected $iZellenSpacing = 0;
		protected $iZeilenSpacingInnen = 0;
		protected $aShowFields;
		protected $aFieldHeadlines;
		protected $aFieldWidth;
		protected $aSchriftausrichtungTextzeile;
		
		/**
		 * Seperater Konstruktor um Listenlimitierung zu berschreiben.
		 *
		 * @param string $sListeName        	
		 * @param bool $bEigenesDrucken
		 *        	- Wenn true wird das Target drucken.php verwendet
		 */
		public function __construct($sListeName, $bEigenesDrucken = false) {
			global $sortierung;
			
			if (strtolower ( $_GET ["format"] ) == "quer") {
				$this->bQuerformat = true;
			}
			
			$sortierung [$this->sListeName] ["anzahl"] = "";
			parent::__construct ( $sListeName, $bEigenesDrucken );
		}
		protected function getOldArrays() {
			for($i = 0; $i < $this->getAnzahlSpalten (); $i ++) {
				$oSpalte = $this->aSpalten [$i];
				
				if ($oSpalte->iBreite < 1 || $this->isButtonSpalte ( $oSpalte )) {
					continue;
				}
				
				if (empty ( $oSpalte->sTitel )) {
					$oSpalte->sTitel = "kein Titel";
				}
				
				$oSpalte->sTitel = html_entity_decode ( $oSpalte->sTitel );
				$oSpalte->sTitel = $this->replaceEuroSign ( $oSpalte->sTitel );
				
				$this->aShowFields [] = $oSpalte->sFeldname;
				$this->aFieldHeadlines [] = $oSpalte->sTitel;
				$this->aFieldWidth [] = $oSpalte->iBreite;
				
				if ($oSpalte->bAusrichtungRechts) {
					$this->aSchriftausrichtungTextzeile [] = "R";
				} else {
					$this->aSchriftausrichtungTextzeile [] = "L";
				}
			}
		}
		protected function replaceEuroSign($sText) {
			return str_replace ( "&euro;", chr ( 128 ), $sText );
		}
		protected function berechneFeldbreiten() {
			$iGesamtbreitePixel = $this->getBreite () - $this->getBreiteVonButtons ();
			
			if ($this->bQuerformat) {
				$iGesamtbreiteMm = 255;
			} else {
				$iGesamtbreiteMm = 170;
			}
			
			for($i = 0; $i < $this->getAnzahlSpalten (); $i ++) {
				$oSpalte = $this->aSpalten [$i];
				
				if ($oSpalte->iBreite < 1 || $this->isButtonSpalte ( $oSpalte )) {
					continue;
				}
				
				$iBreiteNeu = ($oSpalte->iBreite / $iGesamtbreitePixel) * $iGesamtbreiteMm;
				$oSpalte->iBreite = round ( $iBreiteNeu );
			}
		}
		protected function getLayoutInfo() {
			global $standard_layout_liste_hochformat, $standard_layout_liste_querformat, $sql_query, $tbldokumentlayout;
			
			if ($this->bQuerformat) {
				$sLayout = $standard_layout_liste_querformat;
			} else {
				$sLayout = $standard_layout_liste_hochformat;
			}
			
			$query = $sql_query ["dokumentlayoutliste"] . " and $tbldokumentlayout.id = '" . $sLayout . "'";
			$aLayout = get_table_row ( $query );
			
			return $aLayout;
		}
		public function output() {
			global $urlpfad, $dateipfad;
			
			$this->berechneFeldbreiten ();
			$this->bestimmeAusrichtung ();
			
			$this->getOldArrays ();
			$sPfadPDF = $this->createPDF ();
			
			if (isset ( $_SERVER ["HTTPS"] ) && strtolower ( $_SERVER ["HTTPS"] ) != "off") {
				$sProtokoll = "https://";
			} else {
				$sProtokoll = "http://";
			}
			
			$sUrlPdf = $sProtokoll . $urlpfad . substr ( $sPfadPDF, strlen ( $dateipfad ) ) . "?ver=" . microtime ( true );
			echo "<!--PdfUrlOutputStart-->" . $sUrlPdf . "<!--PdfUrlOutputEnde-->";
		}
		
		/**
		 *
		 *
		 * Erzeugt eine PDF Datei und gibt ihren Pfad zurck
		 *
		 * @return string Pfad
		 *        
		 */
		protected function createPDF() {
			global $dateipfad, $tmpoutputordner, $sortierung, $filer, $anzahl_gesamt;
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ hole Layout Infos
			$aLayout = $this->getLayoutInfo ();
			$field = $aLayout;
			
			$layout_vorlagen_ersteseite = unserialize ( $aLayout ["dtVorlagenErsteSeite"] );
			$layout_vorlagen_folgeseiten = unserialize ( $aLayout ["dtVorlagenFolgeSeiten"] );
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ Definiere Seitenrnder
			$typ = "seitenraender";
			include ("functions/function_pdf.php");
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge neues PDF Dokument (nur fr Berechnung von Feldbreiten, je nach Schriftart)
			$pdf = new PDF ();
			$pdf->new_document ( $aLayout ["dtDokumentName"], $ausrichtung );
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge Listenkopf
			$printzeile = 0;
			
			foreach ( $this->aShowFields as $key => $show_field ) {
				$feldinhalte [$show_field] = $this->aFieldHeadlines [$key];
			}
			
			$liste_print [$printzeile] ["zeilentyp"] = "kopfzeile";
			$liste_print [$printzeile] ["zeilennr"] = 1;
			$liste_print [$printzeile] ["zeilenmaxnr"] = 1;
			$liste_print [$printzeile] ["zeilenhoehe"] = round ( $aLayout ["dtStandardSchriftGroesse"] / 1.9, 0 ) + $this->iZeilenSpacingInnen;
			$liste_print [$printzeile] ["zeilen_spacing"] = $this->iZeilenSpacing;
			$liste_print [$printzeile] ["zellen_spacing"] = $this->iZellenSpacing;
			$liste_print [$printzeile] ["feldinhalte"] = $feldinhalte;
			$liste_print [$printzeile] ["feldbreiten"] = $this->aFieldWidth;
			$liste_print [$printzeile] ["formatierungen"] ["hintergrund"] = array_fill ( 0, count ( $this->aShowFields ), 150 );
			$liste_print [$printzeile] ["formatierungen"] ["schriftfarbe"] = array_fill ( 0, count ( $this->aShowFields ), 0 );
			$liste_print [$printzeile] ["formatierungen"] ["rahmenstil"] = array_fill ( 0, count ( $this->aShowFields ), "TB" );
			$liste_print [$printzeile] ["formatierungen"] ["schriftstil"] = array_fill ( 0, count ( $this->aShowFields ), "B" );
			$liste_print [$printzeile] ["formatierungen"] ["schriftausrichtung"] = array_fill ( 0, count ( $this->aShowFields ), "L" );
			$liste_print [$printzeile] ["formatierungen"] ["schriftart"] = $aLayout ["dtStandardSchriftArt"];
			$liste_print [$printzeile] ["formatierungen"] ["schriftgroesse"] = $aLayout ["dtStandardSchriftGroesse"];
			
			$printzeile ++;
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ bearbeite Liste zeilenweise, wenn es der Textfilter erlaubt
			$zeilenzahl = 1;
			if (is_array ( $this->aZeilen )) {
				foreach ( $this->aZeilen as $zeile => $listenzeile ) {
					if ($this->aZeilen [$zeile] ["Textfilter"] != "nomatch") {
						
						// ++++ formatiere Listenfelder (HTML zu PDF) (Schriftart muss gesetzt sein fr Zeilenberechnung)
						foreach ( $this->aShowFields as $key => $show_field ) {
							$listenzeile [$show_field] = $this->getPrintableText ( $listenzeile [$show_field] );
							$listenzeile [$show_field] = $this->replaceEuroSign ( $listenzeile [$show_field] );
							
							$pdf->SetFont ( $aLayout ["dtStandardSchriftArt"], "", $aLayout ["dtStandardSchriftGroesse"] );
							$tabellenzeile [$show_field] = $pdf->format_tabellenfeld ( $listenzeile [$show_field], $this->aFieldWidth [$key] );
						}
						
						// ++++ berechne grte Zeilenanzahl der Listenfelder
						$maxzeilen = $pdf->get_maxzeilen ( $tabellenzeile );
						
						// ++++ erzeuge neues Array mit jeweils einer Textzeile
						for($n = 0; $n < $maxzeilen; $n ++) {
							
							foreach ( $this->aShowFields as $key => $show_field ) {
								$feldinhalte [$show_field] = $pdf->extract_print_zeile ( $tabellenzeile [$show_field], $n );
							}
							
							$liste_print [$printzeile] ["zeilentyp"] = "zeile";
							$liste_print [$printzeile] ["zeilennr"] = $n + 1;
							$liste_print [$printzeile] ["zeilenmaxnr"] = $maxzeilen;
							$liste_print [$printzeile] ["zeilenhoehe"] = round ( $aLayout ["dtStandardSchriftGroesse"] / 1.9, 0 ) + $this->iZeilenSpacingInnen;
							$liste_print [$printzeile] ["zeilen_spacing"] = $this->iZeilenSpacing;
							$liste_print [$printzeile] ["zellen_spacing"] = $this->iZellenSpacing;
							$liste_print [$printzeile] ["feldinhalte"] = $feldinhalte;
							$liste_print [$printzeile] ["feldbreiten"] = $this->aFieldWidth;
							
							if ($zeilenzahl % 2 == 0) {
								$liste_print [$printzeile] ["formatierungen"] ["hintergrund"] = array_fill ( 0, count ( $this->aShowFields ), 220 );
							} else {
								$liste_print [$printzeile] ["formatierungen"] ["hintergrund"] = array_fill ( 0, count ( $this->aShowFields ), 255 );
							}
							
							$liste_print [$printzeile] ["formatierungen"] ["schriftfarbe"] = array_fill ( 0, count ( $this->aShowFields ), 0 );
							$liste_print [$printzeile] ["formatierungen"] ["rahmenstil"] = array_fill ( 0, count ( $this->aShowFields ), "" );
							$liste_print [$printzeile] ["formatierungen"] ["schriftstil"] = array_fill ( 0, count ( $this->aShowFields ), "" );
							$liste_print [$printzeile] ["formatierungen"] ["schriftausrichtung"] = $this->aSchriftausrichtungTextzeile;
							$liste_print [$printzeile] ["formatierungen"] ["schriftart"] = $aLayout ["dtStandardSchriftArt"];
							$liste_print [$printzeile] ["formatierungen"] ["schriftgroesse"] = $aLayout ["dtStandardSchriftGroesse"];
							
							$printzeile ++;
						}
						$zeilenzahl ++;
					}
				}
			}
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge Listenfu
			
			$liste_print [$printzeile] = $liste_print [0];
			$liste_print [$printzeile] ["zeilentyp"] = "fusszeile";
			$liste_print [$printzeile] ["feldinhalte"] = array (
					"Anzahl: " . $this->getAnzahlZeilenAusgabe () . " von " . $anzahl_gesamt 
			);
			$liste_print [$printzeile] ["feldbreiten"] = array (
					($this->getBreite () - $this->getBreiteVonButtons ()) 
			);
			
			$printzeile ++;
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ schliesse temporres PDF Dokument
			$pdf->Close ();
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ fge Seitenumbrche ein
			$liste_print = $pdf->tabellenseitenumbrueche ( $liste_print, round ( $aLayout ["dtStandardSchriftGroesse"] / 1.9, 0 ), $druckhoehe_p1, $druckhoehe );
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ drucke alle Seiten
			$pdf->tabelle_output ( $liste_print, $rand_links_p1, $rand_oben_p1, $rand_links, $rand_oben, $druckbreite_p1, $druckbreite, $dateiname, $aLayout ["dtDokumentName"], $ausrichtung );
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge ggf. Seite mit Infofeld auf erster Seite
			if ($aLayout ["dtInfofeldVorhanden"] == 1) {
				$infofeld_array = create_infofeld_liste_array ( ucfirst ( $this->sListeName ), $aLayout ["dtInfofeldSchriftArt"], $aLayout ["dtInfofeldSchriftGroesse"], $filter [$this->sListeName], $sortierung [$this->sListeName], $this->aFieldHeadlines, $this->aShowFields );
				create_infofeldseite ( $infofeld_array, $ausrichtung, $aLayout ["dtInfofeldLinks"], $aLayout ["dtInfofeldOben"], $seitenbreite - $aLayout ["dtInfofeldLinks"] - $aLayout ["dtInfofeldRechts"], $seitenhoehe - $aLayout ["dtInfofeldOben"] - $aLayout ["dtInfofeldUnten"] );
			}
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge ggf. Seite mit Datumsfeld auf erster Seite
			if ($aLayout ["dtDatumsfeldVorhanden"] == 1) {
				create_datumsfeldseite ( $aLayout ["dtDatumsfeldText"], $ausrichtung, $aLayout ["dtDatumsfeldLinks"], $aLayout ["dtDatumsfeldOben"], $seitenbreite - $aLayout ["dtDatumsfeldLinks"] - $aLayout ["dtDatumsfeldRechts"], $aLayout ["dtDatumsfeldSchriftArt"], $aLayout ["dtDatumsfeldSchriftGroesse"] );
			}
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ erzeuge ggf. Seiten mit Seitenzahl-Feld fr alle Seiten
			if ($aLayout ["dtSeitenzahlfeldVorhanden"] == 1) {
				create_seitenzahlseiten ( $aLayout ["dtSeitenzahlfeldText"], $ausrichtung, $aLayout ["dtSeitenzahlfeldLinks"], $aLayout ["dtSeitenzahlfeldOben"], $seitenbreite - $aLayout ["dtSeitenzahlfeldLinks"] - $aLayout ["dtSeitenzahlfeldRechts"], $aLayout ["dtSeitenzahlfeldSchriftArt"], $aLayout ["dtSeitenzahlfeldSchriftGroesse"] );
			}
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ hinzufgen des Layouts und zusammenfhren aller Seiten
			$output_filename = create_document ( $layout_vorlagen_ersteseite, $layout_vorlagen_folgeseiten );
			
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// ++++ Ausgabe des PDF im Browser durch Reload der Seite
			return $output_filename;
		}
	}
}

if (! class_exists ( "Listenspalte" )) {
	class Listenspalte {
		public $sTitel;
		public $sFeldname;
		public $iBreite;
		public $bAusrichtungRechts = false; // False = links, True = rechts
		public $bAusrichtungFix = false;
		public $bNichtSortieren = false;
		public $sSortierFeld;
		public $sFarbe;
		public function __construct($sTitel, $sFeldname, $iBreite, $bNichtSortieren = false, $sSortierFeld = "") {
			$this->sTitel = $sTitel;
			$this->sFeldname = $sFeldname;
			$this->iBreite = $iBreite;
			$this->bNichtSortieren = $bNichtSortieren;
			
			if (empty ( $sSortierFeld )) {
				$this->sSortierFeld = $sFeldname;
			} else {
				$this->sSortierFeld = $sSortierFeld;
			}
		}
	}
}

if (! class_exists ( "ListenspalteButton" )) {
	class ListenspalteButton extends Listenspalte {
		public function __construct($sFeldname, $iBreite) {
			$this->sTitel = $sFeldname;
			$this->sFeldname = $sFeldname;
			$this->iBreite = $iBreite;
			$this->bNichtSortieren = true;
			$this->sSortierFeld = $sFeldname;
		}
	}
}

if (! function_exists ( "saveListenspalteBreite" )) {
	function saveListenspalteBreite($iUserId, $sSpalteName, $iSpalteBreite, $sListeName) {
		global $sql_query, $tblsession;
		
		$aSessionRow = get_table_row ( $sql_query ["sessiondatenliste"] . " and $tblsession.id = '" . $iUserId . "'" );
		
		if (! empty ( $aSessionRow ['dtArraySpaltenbreiten'] )) {
			$aSpaltenbreiten = unserialize ( $aSessionRow ['dtArraySpaltenbreiten'] );
		} else {
			$aSpaltenbreiten = array ();
		}
		
		if (! is_array ( $aSpaltenbreiten [$sListeName] )) {
			$aSpaltenbreiten [$sListeName] = array ();
		}
		
		$aSpaltenbreiten [$sListeName] [$sSpalteName] = $iSpalteBreite;
		
		$aSessionRow ['dtArraySpaltenbreiten'] = serialize ( $aSpaltenbreiten );
		update_into_table ( $tblsession, $iUserId, $aSessionRow );
	}
}

if (! function_exists ( "resetListenspalteBreite" )) {
	function resetListenspalteBreite($iUserId, $sListeName) {
		global $sql_query, $tblsession;
		
		$aSessionRow = get_table_row ( $sql_query ["sessiondatenliste"] . " and $tblsession.id = '" . $iUserId . "'" );
		
		if (! empty ( $aSessionRow ['dtArraySpaltenbreiten'] )) {
			$aSpaltenbreiten = unserialize ( $aSessionRow ['dtArraySpaltenbreiten'] );
		} else {
			$aSpaltenbreiten = array ();
		}
		
		if (is_array ( $aSpaltenbreiten [$sListeName] )) {
			unset ( $aSpaltenbreiten [$sListeName] );
			
			$aSessionRow ['dtArraySpaltenbreiten'] = serialize ( $aSpaltenbreiten );
			update_into_table ( $tblsession, $iUserId, $aSessionRow );
		}
	}
}

if (! function_exists ( "saveListenHoeheEinstellung" )) {
	function saveListenHoeheEinstellung($iUserId, $sListeName, $sMaximiereHoehe) {
		global $sql_query, $tblsession;
		
		$aSessionRow = get_table_row ( $sql_query ["sessiondatenliste"] . " and $tblsession.id = '" . $iUserId . "'" );
		
		if (! empty ( $aSessionRow ['dtArrayListenoptionen'] )) {
			$aListenoptionen = unserialize ( $aSessionRow ['dtArrayListenoptionen'] );
		} else {
			$aListenoptionen = array ();
		}
		
		if (! is_array ( $aListenoptionen [$sListeName] )) {
			$aListenoptionen [$sListeName] = array ();
		}
		
		if ($sMaximiereHoehe == "1") {
			$bMaximiereHoehe = true;
		} else {
			$bMaximiereHoehe = false;
		}
		
		$aListenoptionen [$sListeName] ["bMaximiereHoehe"] = $bMaximiereHoehe;
		
		$aSessionRow ['dtArrayListenoptionen'] = serialize ( $aListenoptionen );
		update_into_table ( $tblsession, $iUserId, $aSessionRow );
	}
}

if (! function_exists ( "getListeObject" )) {
	function getListeObject() {
		$aArg = func_get_args ();
		
		switch (strtolower ( $_GET ['liste_output'] )) {
			case "csv" :
				$oRet = new ListeCSV ( $aArg [0], $aArg [1] );
				break;
			
			case "pdf" :
				$oRet = new ListePDF ( $aArg [0], $aArg [1] );
				break;
			
			default :
				$oRet = new ListeHTML ( $aArg [0], $aArg [1] );
				break;
		}
		
		return $oRet;
	}
}

// ***************************************************************************************************
// ************************************* listenkopf ***********************************
//
if (! function_exists ( "listenkopf" )) {
	function listenkopf($listenname, $show_fields, $field_headlines, $field_width, $field_sort_options, $sortierung, $target, $typ, $id = "", $subtarget = "", $subtyp = "", $subid = "") {
		include ($GLOBALS ['include_prefix'] . "config/config.php");
		include ($GLOBALS ['include_prefix'] . "config/config-sqlserver.php");
		
		$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
		
		echo "<table border=0 cellspacing=0 cellpadding=1 class=\"text\" style=\"background-color: " . $bgcolor_formular . ";\" width=\"100%\">";
		echo "<tr><td class=\"formularrahmen_oben\"></td></tr>";
		echo "<tr><td>";
		
		echo "<table border=0 cellspacing=0 cellpadding=1 class=\"text\">";
		echo "<tr style=\"background-color: " . $zeilenhintergrund_listenkopf . ";\">";
		
		for($feld = 0; $feld < count ( $show_fields ); $feld ++) {
			
			if ($field_sort_options [$feld] == "") {
				
				if ($sortierung ["feldname"] == $show_fields [$feld]) {
					if ($sortierung ["richtung"] == "") {
						$richtung = "desc";
						$richtungzeichen = $_SESSION ['char_pfeil_ab'] . $_SESSION ['char_pfeil_ab'] . $_SESSION ['char_pfeil_ab'];
						$richtungname = "aufsteigend";
					} else {
						$richtung = "";
						$richtungzeichen = $_SESSION ['char_pfeil_auf'] . $_SESSION ['char_pfeil_auf'] . $_SESSION ['char_pfeil_auf'];
						$richtungname = "absteigend";
					}
				} elseif ($sortierung ["feldname2"] == $show_fields [$feld]) {
					if ($sortierung ["richtung2"] == "") {
						$richtung = "desc";
						$richtungzeichen = $_SESSION ['char_pfeil_ab'] . $_SESSION ['char_pfeil_ab'];
						$richtungname = "aufsteigend";
					} else {
						$richtung = "";
						$richtungzeichen = $_SESSION ['char_pfeil_auf'] . $_SESSION ['char_pfeil_auf'];
						$richtungname = "absteigend";
					}
				} elseif ($sortierung ["feldname3"] == $show_fields [$feld]) {
					if ($sortierung ["richtung3"] == "") {
						$richtung = "desc";
						$richtungzeichen = $_SESSION ['char_pfeil_ab'];
						$richtungname = "aufsteigend";
					} else {
						$richtung = "";
						$richtungzeichen = $_SESSION ['char_pfeil_auf'];
						$richtungname = "absteigend";
					}
				} else {
					$richtung = "";
					$richtungzeichen = "";
					$richtungname = "absteigend";
				}
				
				echo "<td style=\"width: " . ($field_width [$feld] + 2) . "px; vertical-align: top;\" nowrap><nobr><b>";
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel;
				echo "&sortierung_name=" . $listenname . "&sortierung_feld=" . $show_fields [$feld] . "\" ";
				echo "title=\"nach diesem Feld sortieren\">" . $field_headlines [$feld] . "</a>";
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel;
				echo "&sortierung_name=" . $listenname . "&sortierung_feld=" . $show_fields [$feld] . "&sortierung_richtung=" . $richtung . "\" ";
				echo "title=\"nach diesem Feld " . $richtungname . " sortieren\">" . $richtungzeichen . "</a>";
				echo "</nobr></b></td>";
			} elseif ($field_sort_options [$feld] == "Buttons") {
				
				echo "<td style=\"width: " . ($field_width [$feld] + 2) . "px; vertical-align: top;\" nowrap><nobr><b>";
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&standardsortierungladen=1&sortierung_name=" . $listenname . "\" ";
				echo "title=\"Standard-Sortierung wiederherstellen\">" . $_SESSION ['char_pfeil_links'] . $_SESSION ['char_pfeil_links'] . "</a> ";
				echo "&nbsp;&nbsp;&nbsp;";
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&standardsortierungspeichern=1&sortierung_name=" . $listenname . "\" ";
				echo "title=\"Diese Sortierung als Standard speichern\" ";
				echo "onClick=\"return confirm('Wollen Sie diese Sortierung wirklich als Standard speichern?')\">S</a> ";
				echo "</nobr></b></td>";
			} elseif ($field_sort_options [$feld] == "DruckenButton") {
				
				echo "<td style=\"width: " . $field_width [$feld] . "px; vertical-align: top;\" nowrap><nobr><b>";
				echo "<a href=\"javascript:PDFOutput('pdfoutput.php?target=" . $target . "&typ=drucken')\" ";
				echo "title=\"Diese Liste drucken\">D</a> ";
				echo "</nobr></b></td>";
			} else {
				
				echo "<td style=\"width: " . ($field_width [$feld] + 2) . "px; vertical-align: top;\" nowrap><nobr><b>";
				echo $field_headlines [$feld];
				echo "</nobr></b></td>";
			}
		}
		
		echo "</tr>";
		echo "</table>";
		
		echo "</td></tr>";
		echo "</table>";
	}
}

// ***************************************************************************************************
// ************************************* liste_output ***********************************
//
if (! function_exists ( "liste_output" )) {
	function liste_output($liste_output, $show_fields, $field_width, $field_colors, $height, $target, $typ, $id = "", $subtarget = "", $subtyp = "", $subid = "", $field_parameter = "") {
		include ($GLOBALS ['include_prefix'] . "config/config.php");
		include ($GLOBALS ['include_prefix'] . "config/config-sqlserver.php");
		
		$zeilenzahl = 0;
		$zeilenhoehe = 19;
		$klassen_ersetzungsarray = array (
				"class=\"zuruecksetzenbutton\"",
				"class=\"aenderungszeiger\"",
				"class=\"klickbutton\"" 
		);
		$klassen_ersetzungsarray_neu = array (
				"class=\"zuruecksetzenbutton_liste\"",
				"class=\"aenderungszeiger_liste\"",
				"class=\"klickbutton_liste\"" 
		);
		
		if (! isset ( $GLOBALS ['ajaxtarget'] )) {
			echo "<div style=\"height: " . $height . "px; overflow: auto; background-color: " . $GLOBALS ['zeilenhintergrund2'] . ";\">";
		}
		
		for($zeile = 0; $zeile < count ( $liste_output ); $zeile ++) {
			
			if ($liste_output [$zeile] ["Textfilter"] != "nomatch") {
				
				// div/tr Namen bestimmen
				if (isset ( $liste_output [$zeile] ["id"] )) {
					if (substr ( $liste_output [$zeile] ["id"], 0, 4 ) == "<!--") {
						$id = ( int ) str_replace ( array (
								"<!--",
								"//-->" 
						), "", $liste_output [$zeile] ["id"] );
						$idstring = "id=\"row_id_" . $id . "\"";
					}
				}
				
				if (! isset ( $GLOBALS ['ajaxtarget'] )) {
					$zeilenzahl ++;
				} else {
					$zeilenzahl = $GLOBALS ['zeilenzahl'];
				}
				
				if ($zeilenzahl % 2 == 0) {
					$zeilenhintergrund = $GLOBALS ['zeilenhintergrund2'];
				} else {
					$zeilenhintergrund = $GLOBALS ['zeilenhintergrund1'];
				}
				
				if (count ( $liste_output ) == 1) {
					$formname = rand ( 1, 9000 );
				} else {
					$formname = $zeile;
				}
				
				if (! isset ( $GLOBALS ['ajaxtarget'] )) {
					echo "<form name=\"form" . $formname . "\" action=\"" . $_SERVER ["PHP_SELF"] . "\" enctype=\"multipart/form-data\" method=\"post\" style=\"display:inline;\">";
					echo "<input type=\"hidden\" name=\"zeilenzahl\" VALUE=\"" . $zeilenzahl . "\">";
					echo "<div $idstring>";
				}
				
				echo "<table border=0 cellspacing=0 cellpadding=1 class=\"text\" style=\"table-layout: fixed;\">";
				echo "<tr style=\"background-color: " . $zeilenhintergrund . ";\">";
				
				for($feld = 0; $feld < count ( $show_fields ); $feld ++) {
					
					$check_feld = strip_tags ( $liste_output [$zeile] [$show_fields [$feld]] );
					if (is_numeric ( $check_feld ) or ! (strpos ( $check_feld, "text-align: right" ) === false) or is_numeric ( str_replace ( array (
							" ",
							",",
							":",
							"&euro;",
							"%" 
					), "", $check_feld ) )) {
						$style_align = "text-align: right;";
						$inner_field_width = $field_width [$feld] - 10;
					} else {
						$style_align = "";
						$inner_field_width = $field_width [$feld];
					}
					echo "<td style=\"width: " . ($field_width [$feld] + 2) . "px; vertical-align: top;\" nowrap " . $field_parameter [$feld] . ">";
					
					$liste_output [$zeile] [$show_fields [$feld]] = str_replace ( $klassen_ersetzungsarray, $klassen_ersetzungsarray_neu, $liste_output [$zeile] [$show_fields [$feld]] );
					
					if (strpos ( strtolower ( $liste_output [$zeile] [$show_fields [$feld]] ), "<input" ) !== false) {
						echo $liste_output [$zeile] [$show_fields [$feld]];
					} else {
						echo "<div style=\"width: " . ($inner_field_width) . "px;  vertical-align: top; overflow: hidden;$style_align\">";
						echo $liste_output [$zeile] [$show_fields [$feld]];
						echo "</div>";
					}
					echo "</td>";
				}
				
				echo "</tr>";
				echo "</table>";
				
				if (! isset ( $GLOBALS ['ajaxtarget'] )) {
					echo "</div>";
					echo "</form>";
				}
			}
		}
		
		if (! isset ( $GLOBALS ['ajaxtarget'] )) {
			echo "</div>";
		}
		
		return ($zeilenzahl);
	}
}

// ***************************************************************************************************
// ************************************* liste_output_textfilter ***********************************
//
if (! function_exists ( "liste_output_textfilter" )) {
	function liste_output_textfilter($zeile, $aktive_felder, $textsuche, $textmarker = 1) {
		include ($GLOBALS ['include_prefix'] . "config/config.php");
		include ($GLOBALS ['include_prefix'] . "config/config-sqlserver.php");
		
		unset ( $textmatch );
		if ($textsuche != "") {
			$arr_textsuche = explode ( " ", strToLower ( $textsuche ) );
			for($n = 0; $n < count ( $arr_textsuche ); $n ++) {
				if ($arr_textsuche [$n] != "") {
					$textmatch [$arr_textsuche [$n]] = 0;
					foreach ( ( array ) $aktive_felder as $key => $feldname ) {
						
						$zieltext = $zeile [$feldname];
						
						while ( ! (strpos ( $zieltext, "<" ) === false) ) {
							$startpos = strpos ( $zieltext, "<" );
							$endpos = strpos ( substr ( $zieltext, $startpos ), ">" ) + 1 + $startpos;
							$zieltext = substr ( $zieltext, 0, $startpos ) . substr ( $zieltext, $endpos );
						}
						
						if (! (strpos ( strToLower ( $zieltext ), $arr_textsuche [$n] ) === false)) {
							$textmatch [$arr_textsuche [$n]] ++;
							
							$teil1 = substr ( $zeile [$feldname], 0, strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ) );
							$teil2 = substr ( $zeile [$feldname], strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ), strlen ( $arr_textsuche [$n] ) );
							$teil3 = substr ( $zeile [$feldname], strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ) + strlen ( $arr_textsuche [$n] ) );
							
							if ($textmarker == 1) {
								$zeile [$feldname] = $teil1 . "###--" . $teil2 . "--###" . $teil3;
							} else {
								$zeile [$feldname] = $teil1 . $teil2 . $teil3;
							}
						}
					}
				}
			}
			
			if (in_array ( "0", $textmatch )) {
				$zeile ["Textfilter"] = "nomatch";
			}
		}
		
		if ($textmarker == 1) {
			$zeile = str_replace ( "###--", "<font color=" . $textfilter_match_color . ">", $zeile );
			$zeile = str_replace ( "--###", "</font>", $zeile );
		}
		
		return ($zeile);
	}
}

// ***************************************************************************************************
// ************************************* liste_output_textfilter_markieren ***********************************
//
if (! function_exists ( "liste_output_textfilter_markieren" )) {
	function liste_output_textfilter_markieren($zeile, $aktive_felder, $textsuche, $textmarker = 1) {
		include ($GLOBALS ['include_prefix'] . "config/config.php");
		include ($GLOBALS ['include_prefix'] . "config/config-sqlserver.php");
		
		unset ( $textmatch );
		if ($textsuche != "") {
			$arr_textsuche = explode ( " ", strToLower ( $textsuche ) );
			for($n = 0; $n < count ( $arr_textsuche ); $n ++) {
				if ($arr_textsuche [$n] != "") {
					$textmatch [$arr_textsuche [$n]] = 0;
					foreach ( ( array ) $aktive_felder as $key => $feldname ) {
						
						$zieltext = $zeile [$feldname];
						
						while ( ! (strpos ( $zieltext, "<" ) === false) ) {
							$startpos = strpos ( $zieltext, "<" );
							$endpos = strpos ( substr ( $zieltext, $startpos ), ">" ) + 1 + $startpos;
							$zieltext = substr ( $zieltext, 0, $startpos ) . substr ( $zieltext, $endpos );
						}
						
						if (! (strpos ( strToLower ( $zieltext ), $arr_textsuche [$n] ) === false)) {
							$textmatch [$arr_textsuche [$n]] ++;
							
							$teil1 = substr ( $zeile [$feldname], 0, strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ) );
							$teil2 = substr ( $zeile [$feldname], strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ), strlen ( $arr_textsuche [$n] ) );
							$teil3 = substr ( $zeile [$feldname], strpos ( strToLower ( $zeile [$feldname] ), $arr_textsuche [$n] ) + strlen ( $arr_textsuche [$n] ) );
							
							if ($textmarker == 1) {
								$zeile [$feldname] = $teil1 . "###--" . $teil2 . "--###" . $teil3;
							} else {
								$zeile [$feldname] = $teil1 . $teil2 . $teil3;
							}
						}
					}
				}
			}
		}
		
		if ($textmarker == 1) {
			$zeile = str_replace ( "###--", "<font color=" . $textfilter_match_color . ">", $zeile );
			$zeile = str_replace ( "--###", "</font>", $zeile );
		}
		
		return ($zeile);
	}
}
function erstelleFormularrahmenOben() {
	$html = "<table border=0 cellspacing=0 cellpadding=1 class=\"text\" style=\"background-color: " . $bgcolor_formular . ";\" width=\"100%\">";
	$html .= "<tr><td class=\"formularrahmen_oben\"></td></tr>";
	$html .= "</table>";
	
	return $html;
}
function erstelleFormularrahmenUnten() {
	$html = "<table border=0 cellspacing=0 cellpadding=1 class=\"text\" style=\"background-color: " . $bgcolor_formular . ";\" width=\"100%\">";
	$html .= "<tr><td class=\"formularrahmen_unten\"></td></tr>";
	$html .= "</table>";
	
	return $html;
}

// ***************************************************************************************************
// ************************************* listenfuss ***********************************
//
if (! function_exists ( "listenfuss" )) {
	function listenfuss($listenname, $field_width, $sortierung, $anzahl_anzeige, $anzahl_ausgabe, $anzahl_gesamt, $target, $typ, $id = "", $subtarget = "", $subtyp = "", $subid = "") {
		include ($GLOBALS ['include_prefix'] . "config/config.php");
		include ($GLOBALS ['include_prefix'] . "config/config-sqlserver.php");
		
		$ziel = zielstring ( $target, $typ, $id, $subtarget, $subtyp, $subid );
		
		$breite = array_sum ( $field_width ) + (2 * count ( $field_width ));
		
		$breite_links = 400;
		$breite_rechts = 200;
		$breite_mitte = $breite - $breite_links - $breite_rechts;
		
		$anzahl_output = "&nbsp;&nbsp;Auswahl: " . $anzahl_ausgabe . " von " . $anzahl_gesamt;
		if ($sortierung ["anzahl"] != "") {
			$position_ende = (min ( $anzahl_ausgabe, ($sortierung ["position"] + $sortierung ["anzahl"] - 1) ));
			$anzahl_output .= "&nbsp;&nbsp;|&nbsp;&nbsp;Anzeige: " . $anzahl_anzeige . " Eintrge";
			if ($position_ende > 0) {
				$anzahl_output .= "&nbsp;&nbsp;|&nbsp;&nbsp;Position von " . $sortierung ["position"] . " bis " . $position_ende;
			}
		}
		
		echo "<table border=0 cellspacing=0 cellpadding=1 class=\"text\" style=\"background-color: " . $bgcolor_formular . ";\" width=\"100%\">";
		echo "<tr><td>";
		
		echo "<table border=0 cellspacing=0 cellpadding=0 class=\"text\">";
		
		echo "<tr style=\"background-color: " . $zeilenhintergrund_listenkopf . ";\">";
		echo "<td style=\"width: " . $breite_links . "px; vertical-align: middle;\" nowrap><b>" . $anzahl_output . "</b></td>";
		echo "<td style=\"width: " . $breite_mitte . "px; vertical-align: middle;\" nowrap>&nbsp; </td>";
		
		echo "<form name=\"anzeigebegrenzung\" action=\"$PHP_SELF\" method=\"post\">";
		echo "<input type=\"hidden\" name=\"" . session_name () . "\" value=\"" . session_id () . "\">";
		echo "<input type=\"hidden\" name=\"target\" value=\"" . $target . "\">";
		echo "<input type=\"hidden\" name=\"typ\" value=\"" . $typ . "\">";
		
		echo "<td style=\"width: " . $breite_rechts . "px; vertical-align: middle; text-align: right\" nowrap>";
		
		if ($sortierung ["anzahl"] != "") {
			
			if ($sortierung ["position"] != "1") {
				
				// springe zu Position 1
				$position_von = 1;
				$position_bis = min ( $anzahl_anzeige, ($position_von + $sortierung ["anzahl"] - 1) );
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_position=" . $position_von . "\" ";
				echo "title=\"Springe zum Anfang; Zeige " . $position_von . " bis " . $position_bis . "\">";
				echo "<b>&lt;&lt;</b></a>&nbsp;&nbsp;";
				
				// springe zu Position - Anzahl
				$position_von = max ( 1, $sortierung ["position"] - $sortierung ["anzahl"] );
				$position_bis = min ( $anzahl_anzeige, ($position_von + $sortierung ["anzahl"] - 1) );
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_position=" . $position_von . "\" ";
				echo "title=\"Zeige " . $position_von . " bis " . $position_bis . "\">";
				echo "<b>&lt;</b></a>";
			} else {
				
				echo "<font color=\"" . $zeilenhintergrund_listenkopf . "\"><b>&lt;&lt;</b></a>&nbsp;&nbsp;<b>&lt;</b></font>";
			}
			
			echo "&nbsp;&nbsp;|&nbsp;&nbsp;";
			
			if ($sortierung ["position"] <= ($anzahl_ausgabe - $sortierung ["anzahl"])) {
				
				// springe zu Position + Anzahl
				$position_von = max ( 1, min ( ($anzahl_ausgabe - $sortierung ["anzahl"] + 1), ($sortierung ["position"] + $sortierung ["anzahl"]) ) );
				$position_bis = min ( $anzahl_ausgabe, ($position_von + $sortierung ["anzahl"] - 1) );
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_position=" . $position_von . "\" ";
				echo "title=\"Zeige " . $position_von . " bis " . $position_bis . "\">";
				echo "<b>&gt;</b></a>&nbsp;&nbsp;";
				
				// springe zur letzten Position
				$position_von = max ( 1, $anzahl_ausgabe - $sortierung ["anzahl"] + 1 );
				$position_bis = min ( $anzahl_ausgabe, ($position_von + $sortierung ["anzahl"] - 1) );
				
				echo "<a href=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_position=" . $position_von . "\" ";
				echo "title=\"Sprige zum Ende; Zeige " . $position_von . " bis " . $position_bis . "\">";
				echo "<b>&gt;&gt;</b></a>&nbsp;&nbsp;";
			} else {
				
				echo "<font color=\"" . $zeilenhintergrund_listenkopf . "\"><b>&gt;</b></a>&nbsp;&nbsp;<b>&gt;&gt;</b></a>&nbsp;&nbsp;</font>";
			}
		}
		
		echo "<select size=1 class=\"input_select_50\" name=\"anzeigebegrenzung\" ";
		echo "title=\"Anzeige begrenzen auf ... Eintrge\" ";
		echo "onChange='Go(this.form.anzeigebegrenzung.options[this.form.anzeigebegrenzung.options.selectedIndex].value,0)'>";
		
		$selectbox_content = array (
				"alle" => "",
				"100" => "100",
				"50" => "50",
				"20" => "20",
				"10" => "10" 
		);
		
		foreach ( $selectbox_content as $option_name => $option_value ) {
			
			if (($sortierung ["position"] + $option_value) > $anzahl) {
				$position_neu = max ( 1, $anzahl - $option_value + 1 );
			} else {
				$position_neu = $sortierung ["position"];
			}
			
			if ($sortierung ["anzahl"] == $option_value) {
				echo "<option value=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_anzahl=" . $option_value . "&sortierung_position=" . $position_neu . "\" selected>" . $option_name . "</option>";
			} else {
				echo "<option value=\"main.php?" . SID . "&" . $ziel . "&sortierung_name=" . $listenname . "&sortierung_anzahl=" . $option_value . "&sortierung_position=" . $position_neu . "\">" . $option_name . "</option>";
			}
		}
		echo "</select>";
		
		echo "</td></form></tr>";
		echo "</table>";
		
		echo "</td></tr>";
		echo "<tr><td class=\"formularrahmen_unten\"></td></tr>";
		echo "</table>";
	}
}

if (! function_exists ( "listeanzahl_ermitteln" )) {
	function listeanzahl_ermitteln($sTable) {
		Global $tbllisteanzahl;
		
		$sQuery = "SELECT * FROM " . $tbllisteanzahl . " WHERE dtListeName LIKE '" . $sTable . "'";
		$rListeAnzahl = get_table_row ( $sQuery );
		if (! $rListeAnzahl) {
			$aListeAnzahl ["dtListeName"] = $sTable;
			$aListeAnzahl ["dtAnzahl"] = get_table_count_complete ( $sTable );
			insert_into_table ( $tbllisteanzahl, $aListeAnzahl );
			return $aListeAnzahl ["dtAnzahl"];
		} else {
			return $rListeAnzahl ["dtAnzahl"];
		}
	}
}

if (! function_exists ( "listeanzahl_aendern" )) {
	function listeanzahl_aendern($sTable, $iAnzahl) {
		Global $tbllisteanzahl;
		
		$sQuery = "SELECT * FROM " . $tbllisteanzahl . " WHERE dtListeName LIKE '" . $sTable . "'";
		$rListeAnzahl = get_table_row ( $sQuery );
		
		if ($rListeAnzahl) {
			$rListeAnzahl ["dtAnzahl"] = $rListeAnzahl ["dtAnzahl"] + $iAnzahl;
			$iId = $rListeAnzahl ["id"];
			unset ( $rListeAnzahl ["id"] );
			update_into_table ( $tbllisteanzahl, $iId, $rListeAnzahl );
		}
	}
}

if (! function_exists ( "generateList" )) {
	/**
	 * Read out the Listinformation for creating a ListObject
	 *
	 * @param string $listName        	
	 */
	function generateList($listName) {
		global $tbllist, $tbllistcolumn;
		
		// Create the Listrow for the overall information about the list
		$rList = get_table_row ( "SELECT * FROM $tbllist WHERE dtListName LIKE '%" . $listName . "%'" );
		
		// Create the Listcolumnarray for the column information about the list
		$aListColumn = get_table_array ( "SELECT * FROM $tbllistcolumn WHERE fiListID = " . $rList ["id"] . " ORDER BY dtPos" );
		return createListObject ( $rList ["dtListName"], $aListColumn );
	}
}

if (! function_exists ( "createListObject" )) {
	/**
	 * Create the listobject for the represantion of specific datas in a table
	 *
	 *
	 * @param string $sName        	
	 * @param object $oData        	
	 * @return ListeHTML
	 */
	function createListObject($sName, $oData) {
		$oListe = getListeObject ( $sName, true );
		
		foreach ( $oData as $cpoData ) {
			
			// Check the Exception or go further and create the Listcolumnobject
			if (eval ( $cpoData ["dtException"] ) || $cpoData ["dtException"] == "") {
				// If the column sortable, set it with TRUE
				if ($cpoData ["dtSortable"] == 1)
					$cpoData ["dtSortable"] = TRUE;
				else
					$cpoData ["dtSortable"] = FALSE;
					
					// If the column a button, take another function
				if (! $cpoData ["dtIsButton"])
					$oListe->addSpalte ( new Listenspalte ( $cpoData ["dtColumnDescription"], $cpoData ["dtColumnFieldName"], $cpoData ["dtWidth"], $cpoData ["dtSortable"] ) );
				else
					$oListe->addSpalte ( new ListenspalteButton ( $cpoData ["dtColumnFieldName"], $cpoData ["dtWidth"] ) );
			}
		}

		return $oListe;
	}
}

?>