<?php

/**
 * OX API Klasse
 *
 * Dokumentation: http://oxpedia.org/wiki/index.php?title=HTTP_API
 */
if (! class_exists ( "KontaktOX6" )) {
	class KontaktOX6 {
		protected $iKontaktordner;
		protected $sName;
		protected $sPassword;
		protected $sUrl = "http://localhost";
		protected $rCurl; // Curl-Ressource
		protected $sSession; // OX Session ID
		protected $sCookiefile; // Cookies
		protected $sIdPrefix = "conceptERP-KontaktID-";
		protected $sIdPrefixUnknown = "conceptERP-Unknown-";
		protected $sLastResult;
		public $Ordner;
		
		/**
		 * Konstruktor
		 */
		public function __construct($sName, $sPassword, $sUrl) {
			$this->sName = $sName;
			$this->sPassword = $sPassword;
			$this->sUrl = $sUrl;
			
			$this->rCurl = curl_init ();
			$this->sCookiefile = tempnam ( sys_get_temp_dir (), "conceptERP-Cookiefile-" );
			
			curl_setopt ( $this->rCurl, CURLOPT_COOKIEJAR, $this->sCookiefile );
			curl_setopt ( $this->rCurl, CURLOPT_RETURNTRANSFER, true );
			curl_setopt ( $this->rCurl, CURLOPT_AUTOREFERER, true );
			curl_setopt ( $this->rCurl, CURLOPT_FOLLOWLOCATION, true );
			
			$this->login ();
		}
		
		/**
		 * Destruktor
		 */
		public function __destruct() {
			$this->logout ();
			
			unlink ( $this->sCookiefile );
			curl_close ( $this->rCurl );
		}
		
		/**
		 * Erzeugt eine Session am Server
		 */
		protected function login() {
			$this->resetCurl ();
			curl_setopt ( $this->rCurl, CURLOPT_POSTFIELDS, "name=" . $this->sName . "&password=" . $this->sPassword );
			curl_setopt ( $this->rCurl, CURLOPT_POST, true );
			curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/login?action=login" );
			
			$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
			
			$this->sSession = $oReturn->session;
		}
		
		/**
		 * Beendet die Sitzung
		 */
		protected function logout() {
			$this->resetCurl ();
			curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, true );
			curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/login?action=logout" . $this->createUrlParameters ( array (
					"session" => $this->sSession 
			) ) );
			curl_exec ( $this->rCurl );
		}
		
		/**
		 *
		 * Liefert eine Liste aller Kontaktordner
		 *
		 * @return array
		 */
		public function getOrdnerstruktur() {
			$this->resetCurl ();
			curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, true );
			curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/folders?action=allVisible" . $this->createUrlParameters ( array (
					"session" => $this->sSession,
					"content_type" => "contacts",
					"columns" => "1,300",
					"tree" => "1" 
			) ) );
			
			$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
			$aOrdnerReturn = array ();
			
			foreach ( $oReturn->data->private as $aOrdner ) {
				$iOrdnerId = $aOrdner [0];
				$iOrdnerName = $aOrdner [1];
				
				$aOrdnerReturn [$iOrdnerId] = $iOrdnerName;
			}
			
			return $aOrdnerReturn;
		}
		
		/**
		 *
		 * Setzt den zu verwendenden Kontaktordner
		 *
		 * @param int $iOrdnerId        	
		 */
		public function setKontaktordner($iOrdnerId) {
			if (array_key_exists ( $iOrdnerId, $this->getOrdnerstruktur () )) {
				$this->iKontaktordner = $iOrdnerId;
			} else {
				throw new Exception ( "Der Ordner existiert nicht." );
			}
		}
		
		/**
		 *
		 * Legt einen neuen Kontakt an oder ndert ihn.
		 *
		 * @param object $oKontakt        	
		 * @return int Kontakt-ID
		 */
		public function add($oKontakt) {
			if (empty ( $oKontakt->Vorname ) && empty ( $oKontakt->Name ) && empty ( $oKontakt->Firma ) && empty ( $oKontakt->Namenszusatz )) {
				throw new Exception ( "Kontaktdaten nicht komplett!" );
			}
			
			// Id Prefix ggf. entfernen
			$oKontakt->Id = str_replace ( $this->sIdPrefix, "", $oKontakt->Id );
			
			// Kontaktinformationen
			$aSend = array ();
			$aSend ['userfield01'] = $this->sIdPrefix . $oKontakt->Id;
			$aSend ['last_name'] = $oKontakt->Name;
			$aSend ['first_name'] = $oKontakt->Vorname;
			$aSend ['suffix'] = $oKontakt->Namenszusatz;
			
			$aSend ['telephone_business1'] = $oKontakt->TelNr;
			$aSend ['fax_business'] = $oKontakt->FaxNr;
			$aSend ['cellular_telephone1'] = $oKontakt->MobilNr;
			$aSend ['email1'] = $oKontakt->EMail;
			
			$aSend ['company'] = $oKontakt->Firma;
			$aSend ['position'] = $oKontakt->Position;
			
			$aSend ['street_business'] = $oKontakt->Strasse;
			$aSend ['postal_code_business'] = $oKontakt->PLZ;
			$aSend ['city_business'] = $oKontakt->Ort;
			$aSend ['country_business'] = $oKontakt->Land;
			
			$aSend ['folder_id'] = $this->iKontaktordner;
			$aSend ['private_flag'] = false;
			
			// Anzeigenamen
			if (empty ( $oKontakt->Firma ) && (! empty ( $oKontakt->Name ) || ! empty ( $oKontakt->Vorname ))) {
				$aSend ['display_name'] = $oKontakt->Name . ", " . $oKontakt->Vorname;
			} elseif (! empty ( $oKontakt->Firma ) && empty ( $oKontakt->Name ) && empty ( $oKontakt->Vorname )) {
				$aSend ['display_name'] = $oKontakt->Firma;
			} elseif (! empty ( $oKontakt->Firma ) && (! empty ( $oKontakt->Name ) || ! empty ( $oKontakt->Vorname ))) {
				$aSend ['display_name'] = $oKontakt->Firma . " (" . $oKontakt->Name . ", " . $oKontakt->Vorname . ")";
			}
			
			$sJsonASend = json_encode ( $aSend );
			
			// Bestimme ob ndern oder neu
			if (($iKontaktId = $this->getIdFromErpId ( $oKontakt->Id )) > 0) {
				// ndern
				$this->resetCurl ();
				curl_setopt ( $this->rCurl, CURLOPT_INFILESIZE, strlen ( $sJsonASend ) );
				curl_setopt ( $this->rCurl, CURLOPT_INFILE, $this->getTmpFile ( $sJsonASend ) );
				curl_setopt ( $this->rCurl, CURLOPT_PUT, true );
				curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=update" . $this->createUrlParameters ( array (
						"session" => $this->sSession,
						"folder" => $this->iKontaktordner,
						"timestamp" => $this->getTimestamp (),
						"id" => $iKontaktId 
				) ) );
				
				$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
				
				return $iKontaktId;
			} else {
				// Neu
				$this->resetCurl ();
				curl_setopt ( $this->rCurl, CURLOPT_INFILESIZE, strlen ( $sJsonASend ) );
				curl_setopt ( $this->rCurl, CURLOPT_INFILE, $this->getTmpFile ( $sJsonASend ) );
				curl_setopt ( $this->rCurl, CURLOPT_PUT, true );
				curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=new" . $this->createUrlParameters ( array (
						"session" => $this->sSession 
				) ) );
				$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
				
				return $oReturn->data->id;
			}
		}
		
		/**
		 *
		 * Fgt mehrere Kontakte hinzu
		 *
		 * @param array $aKontakte        	
		 */
		public function addArray($aKontakte) {
			if (! is_array ( $aKontakte )) {
				return;
			}
			
			foreach ( $aKontakte as $oKontakt ) {
				try {
					$this->add ( $oKontakt );
				} 

				catch ( Exception $e ) {
				}
			}
		}
		
		/**
		 *
		 * Kontakt-ID ndern
		 *
		 * @param string $sIdAlt        	
		 * @param string $sIdNeu        	
		 */
		public function aendereID($sIdAlt, $sIdNeu) {
			$aKontakte = $this->get ( $sIdAlt );
			if (count ( $aKontakte ) > 0) {
				$oKontakt = $aKontakte [0];
				$oKontakt->Id = $this->sIdPrefix . $sIdNeu;
				$this->add ( $oKontakt );
				$this->loeschen ( $sIdAlt );
			} else {
				return false;
			}
		}
		
		/**
		 *
		 * ndert die ERP-Id eines Kontakts anhand seiner OX Id
		 *
		 * @param int $iOXid        	
		 * @param string $sIdNeu        	
		 */
		protected function aendereIDbyOXid($iOXid, $sIdNeu) {
			$aSend = array ();
			$aSend ['userfield01'] = $sIdNeu;
			$sJsonASend = json_encode ( $aSend );
			
			$this->resetCurl ();
			curl_setopt ( $this->rCurl, CURLOPT_INFILESIZE, strlen ( $sJsonASend ) );
			curl_setopt ( $this->rCurl, CURLOPT_INFILE, $this->getTmpFile ( $sJsonASend ) );
			curl_setopt ( $this->rCurl, CURLOPT_PUT, true );
			curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=update" . $this->createUrlParameters ( array (
					"session" => $this->sSession,
					"folder" => $this->iKontaktordner,
					"timestamp" => $this->getTimestamp (),
					"id" => $iOXid 
			) ) );
			
			$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
		}
		
		/**
		 *
		 * Kontakt(e) abrufen
		 *
		 * @param string $sErpId        	
		 */
		public function get($sErpId = false) {
			$aAusgabe = array ();
			
			if ($sErpId === false) // Alle Kontakte
{
				$this->resetCurl ();
				curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, true );
				curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=all" . $this->createUrlParameters ( array (
						"session" => $this->sSession,
						"timezone" => $this->getTimezoneOffset (),
						"folder" => $this->iKontaktordner,
						"columns" => "1,501,502,569,520,504,542,551,544,555,523,525,526,528,571,5" 
				) ) );
				
				$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
				
				$iUnknownCount = 0;
				
				foreach ( $oReturn->data as $aReturnKontakt ) {
					$iInterneId = $aReturnKontakt [0];
					
					if (empty ( $aReturnKontakt [14] ) || substr ( $aReturnKontakt [14], 0, strlen ( $this->sIdPrefixUnknown ) ) == $this->sIdPrefixUnknown) {
						$this->aendereIDbyOXid ( $iInterneId, ($aReturnKontakt [14] = $this->sIdPrefixUnknown . $iUnknownCount ++) );
					}
					
					$aErpIdSplit = explode ( "-", $aReturnKontakt [14] );
					$sId = $str_id [3];
					$sTyp = $str_id [2];
					unset ( $aErpIdSplit );
					
					if ($sTyp == "ASP") {
						$oKontakt = new Ansprechpartner ();
					} else {
						$oKontakt = new Kontakt ();
					}
					
					$oKontakt->Id = $aReturnKontakt [14];
					
					$oKontakt->Vorname = $aReturnKontakt [1];
					$oKontakt->Name = $aReturnKontakt [2];
					$oKontakt->Firma = $aReturnKontakt [3];
					$oKontakt->Position = $aReturnKontakt [4];
					$oKontakt->Namenszusatz = $aReturnKontakt [5];
					
					$oKontakt->TelNr = $aReturnKontakt [6];
					$oKontakt->MobilNr = $aReturnKontakt [7];
					$oKontakt->FaxNr = $aReturnKontakt [8];
					$oKontakt->EMail = $aReturnKontakt [9];
					
					$oKontakt->Strasse = $aReturnKontakt [10];
					$oKontakt->PLZ = $aReturnKontakt [11];
					$oKontakt->Ort = $aReturnKontakt [12];
					$oKontakt->Land = $aReturnKontakt [13];
					
					$oKontakt->Unknown = true;
					
					$oKontakt->Aenderungszeit = $this->getMillisekundenZuUnixtime ( $aReturnKontakt [15] );
					
					$aAusgabe [] = $oKontakt;
				}
			} else // Bestimmter Kontakt
{
				$iKontaktId = $this->getIdFromErpId ( $sErpId );
				
				if ($iKontaktId == 0) {
					return $aAusgabe;
				}
				
				$this->resetCurl ();
				curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, true );
				curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=get" . $this->createUrlParameters ( array (
						"session" => $this->sSession,
						"timezone" => $this->getTimezoneOffset (),
						"folder" => $this->iKontaktordner,
						"id" => $iKontaktId 
				) ) );
				
				$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
				
				$aErpIdSplit = explode ( "-", $oReturn->data->userfield01 );
				$sId = $str_id [3];
				$sTyp = $str_id [2];
				unset ( $aErpIdSplit );
				
				if ($sTyp == "ASP") {
					$oKontakt = new Ansprechpartner ();
				} else {
					$oKontakt = new Kontakt ();
				}
				
				$oKontakt->Vorname = $oReturn->data->first_name;
				$oKontakt->Name = $oReturn->data->last_name;
				$oKontakt->Firma = $oReturn->data->company;
				$oKontakt->Position = $oReturn->data->position;
				$oKontakt->Namenszusatz = $oReturn->data->suffix;
				
				$oKontakt->TelNr = $oReturn->data->telephone_business1;
				$oKontakt->MobilNr = $oReturn->data->cellular_telephone1;
				$oKontakt->FaxNr = $oReturn->data->fax_business;
				$oKontakt->EMail = $oReturn->data->email1;
				
				$oKontakt->Strasse = $oReturn->data->street_business;
				$oKontakt->PLZ = $oReturn->data->postal_code_business;
				$oKontakt->Ort = $oReturn->data->city_business;
				$oKontakt->Land = $oReturn->data->country_business;
				
				$oKontakt->Unknown = true;
				
				$oKontakt->Aenderungszeit = $this->getMillisekundenZuUnixtime ( $oReturn->timestamp );
				
				$aAusgabe [] = $oKontakt;
			}
			
			return $aAusgabe;
		}
		
		/**
		 *
		 * Einzelnen Kontakt lschen
		 *
		 * @param string $sErpId        	
		 */
		public function loeschen($sErpId) {
			$iKontaktId = $this->getIdFromErpId ( $sErpId );
			
			if ($iKontaktId > 0) {
				$aDelete = array (
						"id" => $iKontaktId,
						"folder" => $this->iKontaktordner 
				);
				$sJsonADelete = json_encode ( $aDelete );
				
				$this->resetCurl ();
				curl_setopt ( $this->rCurl, CURLOPT_INFILESIZE, strlen ( $sJsonADelete ) );
				curl_setopt ( $this->rCurl, CURLOPT_INFILE, $this->getTmpFile ( $sJsonADelete ) );
				curl_setopt ( $this->rCurl, CURLOPT_PUT, true );
				curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=delete" . $this->createUrlParameters ( array (
						"session" => $this->sSession,
						"timestamp" => $this->getTimestamp () 
				) ) );
				$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
			}
		}
		
		/**
		 *
		 * Viele Kontakte lschen
		 *
		 * @param array $aErpIds        	
		 */
		public function loeschenArray($aErpIds) {
			if (! is_array ( $aErpIds ))
				return;
			
			foreach ( $aErpIds as $sId ) {
				try {
					$this->loeschen ( $sId );
				} 

				catch ( Exception $e ) {
				}
			}
		}
		
		/**
		 *
		 * Gibt den Timezone-Offset zurck, z.B. +0200
		 *
		 * @return string
		 */
		protected function getTimezoneOffset() {
			return date ( "O" );
		}
		
		/**
		 *
		 * Rechnet Millisekunden seit 1970 in Unixtime um.
		 *
		 * @param float $fMillisekunden        	
		 * @return int Unixtime
		 */
		protected function getMillisekundenZuUnixtime($fMillisekunden) {
			return round ( $fMillisekunden / 1000 );
		}
		
		/**
		 *
		 * Durchsucht die Kontaktdatenbank nach einer conceptERP-ID und gibt die Kontakt-ID zurck
		 *
		 * @param string $sErpId        	
		 * @return int Kontakt-ID oder 0
		 */
		protected function getIdFromErpId($sErpId) {
			if (strpos ( $sErpId, $this->sIdPrefix ) !== false || strpos ( $sErpId, $this->sIdPrefixUnknown ) !== false) {
				$sErpId = $sErpId;
			} else {
				$sErpId = $this->sIdPrefix . $sErpId;
			}
			
			$this->resetCurl ();
			curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, true );
			curl_setopt ( $this->rCurl, CURLOPT_URL, $this->sUrl . "/ajax/contacts?action=all" . $this->createUrlParameters ( array (
					"session" => $this->sSession,
					"folder" => $this->iKontaktordner,
					"columns" => "1,571" 
			) ) );
			$oReturn = $this->parseReturn ( curl_exec ( $this->rCurl ) );
			
			foreach ( $oReturn->data as $aErgebnis ) {
				if ($aErgebnis [1] == $sErpId) {
					return $aErgebnis [0];
				}
			}
			
			return 0;
		}
		
		/**
		 *
		 * Verarbeitet einen OX Response und fngt ggf. Fehler ab.
		 *
		 * @param string $sReturn        	
		 * @throws Exception
		 * @return object
		 */
		protected function parseReturn($sReturn) {
			$oReturn = json_decode ( $sReturn );
			
			if (is_null ( $oReturn )) {
				throw new Exception ( "OX Antwort nicht lesbar. (" . $sReturn . ")" );
			}
			
			if (isset ( $oReturn->error )) {
				$e = new Exception ( $oReturn->error . " JSON: (" . $sReturn . ")" );
				throw $e;
			}
			
			// Debugging
			if ($GLOBALS ['bOXDebug']) {
				echo "<hr style='border:2px solid'>";
			}
			
			$this->sLastResult = $oReturn;
			
			return $oReturn;
		}
		
		/**
		 *
		 * Wandelt ein Array in URL-Parameter um
		 *
		 * @param array $aParameters        	
		 * @return string
		 */
		protected function createUrlParameters($aParameters) {
			$sPara = "";
			foreach ( ( array ) $aParameters as $name => $value ) {
				$sPara .= "&" . $name . "=" . urlencode ( $value );
			}
			
			return $sPara;
		}
		
		/**
		 * Setzt Optionen von CURL zurck
		 */
		protected function resetCurl() {
			curl_setopt ( $this->rCurl, CURLOPT_POSTFIELDS, null );
			curl_setopt ( $this->rCurl, CURLOPT_POST, false );
			curl_setopt ( $this->rCurl, CURLOPT_HTTPGET, false );
			curl_setopt ( $this->rCurl, CURLOPT_INFILESIZE, 0 );
			curl_setopt ( $this->rCurl, CURLOPT_PUT, false );
		}
		
		/**
		 *
		 * Erzeugt eine temporre Datei mit dem Inhalt $sContent und gibt die Fileressource zurck.
		 *
		 * @param string $sContent        	
		 * @return resource Tmpfile
		 */
		protected function getTmpFile($sContent) {
			$rFile = tmpfile ();
			fwrite ( $rFile, $sContent );
			rewind ( $rFile );
			return $rFile;
		}
		
		/**
		 *
		 * Erzeugt einen OX-gerechten Timestamp.
		 *
		 * @return int Millisekunden seit 1.1.1970
		 */
		protected function getTimestamp() {
			return round ( microtime ( true ) * 1000 ); // Geht nur Sekundengenau
		}
	}
}

?>